#include "backEndParser.h"
#include "circular_queue.h"
#include "uart.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "type.h"
#include "system.h"
#include "config.h"
#include "timers.h"
#include <math.h>
#include "parser.h"


char ResponseBuf[BE_MAX_CMD_SIZE];
//char myBuffer[BE_MAX_CMD_SIZE];
static char myBETmpBuf[BE_MAX_CMD_SIZE];
char myCmdTok[BE_MAX_CMD_SIZE];
char myOxCmdTok[BE_MAX_CMD_SIZE];
char myOxCmdTokCpy[BE_MAX_CMD_SIZE];
char myOxCmdTokCpy2[BE_MAX_CMD_SIZE];

char myMatchString[BE_MAX_CMD_SIZE];
char* argmt1;
char* argmt2;
char* myCmdPtr;
char* myMatchPtr;
char* tokens[10];
char delim=' ';
ushort tokIndex=0;

unsigned char globalResponseBuffA[64]={0};
unsigned char globalResponseBuffB[64]={0};
unsigned char globalResponseBuffC[64]={0};
unsigned char globalResponseBuffD[64]={0};
unsigned char *globalResponsePtr;

int backEndHandler(void);


extern void strupr(char *myUprCaseBuf, char *myInputBuf);


extern VOLATILE_REGISTERS shadowRegs;
extern char outgoing[256]; //Top Level Output Buffer
extern unsigned char cmdToMatch[5];
extern unsigned char configuring;

extern unsigned char A_Response_Pending;
extern unsigned char B_Response_Pending;
extern unsigned char C_Response_Pending;
extern unsigned char D_Response_Pending;
extern unsigned char GLOBAL_Response_Pending;

//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
//Begin OXXIUS Constants, Structs, Functions
//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

//AL? Returns cause of latest alarm
//BT? Returns laser head case temp in deg C
//DL? Returns "state" of laser module
//ET? Returns control electronics temp in deg C
//IP? Returns laser output power
//TM? Returns laser time in hrs
//VA? Returns Returns DC input voltage
//VE? Returns FW version

const AL_RESPONSE al_Response[]={
{"LASER HEAD TEMPERATURE"},
{"CONTROLLER TEMPERATURE"},
{"INTERLOCK"},
{"POWER SUPPLY"},
{"OTHER"},
{"NO ALARM"},
{""},
};

const DL_Q_RESPONSE dl_q_Response[]={
{"LASER OFF"},
{"SETTING TEMPERATURE"},
{"WAITING FOR STABILIZATION"},
{"STARTING UP"},
{"LASER ON"},
{"LASER ALARM"},
{"POWER LOCK LOST"},
{"FAN WARNING"},
{"POWER SUPPLY WARNING"},
{"TEMPERATURE WARNING"},
{""},
};

const OXXIUS_COMMANDS ox_Cmds[]={
{1,ox_BE_dlOnHandler}, 
{2,ox_BEq_dlfcHandler}, //Returns fault codes
{3,ox_BEq_dlfdHandler}, //Returns fault descriptions
{4,ox_BEq_btHandler},
{5,ox_BEq_dlHandler},
{6,ox_BEq_ipHandler},
{7,ox_BEq_tmHandler},
{8,ox_BEq_veHandler},
{9,ox_BE_invalHandler},
{10,ox_BEq_etHandler},
{11,ox_BE_dlOffHandler},
//12 is reserved
{0,0}
};
const char OK[]={"OK"};
const char INVALID_REPLY[]={"????"};
const char NA[]={"NOT AUTHORIZED"};
const char TEST[]={"1"};

//Cmd String, Max Args, Handler Func Ptr
const VERIFY_CMDS VerifyCmds[] = {
	
	//User Level Commands
	{"C",				1,  BEcHandler},
	{"DELAY", 	        1,  BEdelayHandler},
	{"EPC", 			1,  BEepcHandler},
	{"LE", 			    1,  BEleHandler},
	{"LP", 			    1,  BElpHandler},
	{"TEC", 			1,  BEtecHandler},
	{"VOBTS", 		    1,  BEvobtsHandler},
	{"PUL", 		    1,  BEpulHandler},
	{"PP", 		        1,  BEppHandler},
	{"PROMPT",          1,  BEpromptHandler}, //Only used to detect lasers
	{"?BPT", 			1,  BEq_bptHandler},
	{"?C",				1,  BEq_cHandler},
	{"?CC",				1,	BEq_ccHandler},
	{"?DELAY", 			1,  BEq_delayHandler},
	{"?EPC", 			1,  BEq_epcHandler},
	{"?FC", 			1,  BEq_fcHandler},
	{"?FD", 			1,  BEq_fdHandler},
	{"?FP", 			1,  BEq_fpHandler},
	{"?FV", 			1,  BEq_fvHandler},
	{"?IL", 			1,  BEq_ilHandler},
	{"?LC", 			1,  BEq_lcHandler},
	{"?LE", 			1,  BEq_leHandler},
	{"?LH", 			1,	BEq_lhHandler},
	{"?LI", 			1,	BEq_liHandler},
	{"?LP", 			1,	BEq_lpHandler},
	{"?LPS", 			1,	BEq_lpsHandler},
	{"?LW", 			1,	BEq_lwHandler},
	{"?MAXP", 			1,	BEq_maxpHandler},
	{"?OBT",			1,	BEq_obtHandler},
	{"?OBTS", 			1,	BEq_obtsHandler},
	{"?RP", 			1,	BEq_rpHandler},
	{"?TEC", 			1,	BEq_tecHandler},
	{"?PUL", 			1,	BEq_pulHandler},
	{"?PP", 			1,	BEq_ppHandler},
	{"?OP1",            1,  BEq_op1Handler},
	{"?OP2",            1,  BEq_op2Handler},
	{"?OP3",            1,  BEq_op3Handler},
	{"?OP4",            1,  BEq_op4Handler},

	{"",0,0} // DO NOT MOVE/REMOVE!!	END OF COMMANDS!!
};

void ox_BE_dlOnHandler(void)
{
	int tempLE=0;

	//Returns "OK" when turn on OR turn off is successful
	if(strcmp(tokens[0],OK)==0)
	{
		tempLE=1;
	}
	else
		tempLE=0;

	if(portToDevice==UART_1)
	{
		A_Response_Pending=0;
		sprintf(outgoing, "\r\nA.LE=%d", tempLE);
		sendString(portToHost,outgoing);				
	}
	else
		if(portToDevice==UART_2)
		{
			B_Response_Pending=0;
			sprintf(outgoing, "\r\nB.LE=%d", tempLE);
			sendString(portToHost,outgoing);						
		}
		else
			if(portToDevice==UART_3)
			{
				C_Response_Pending=0;
				sprintf(outgoing, "\r\nC.LE=%d", tempLE);
				sendString(portToHost,outgoing);					
			}
			else
				if(portToDevice==UART_4)
				{
					D_Response_Pending=0;
					sprintf(outgoing, "\r\nD.LE=%d", tempLE);
					sendString(portToHost,outgoing);					
				}
	
	composePrompt();
}

void ox_BE_dlOffHandler(void)
{
	int tempLE=0;
		
	tempLE=0; //Successfully turned off


	if(portToDevice==UART_1)
	{
		A_Response_Pending=0;
		sprintf(outgoing, "\r\nA.LE=%d", tempLE);
		sendString(portToHost,outgoing);				
	}
	else
		if(portToDevice==UART_2)
		{
			B_Response_Pending=0;
			sprintf(outgoing, "\r\nB.LE=%d", tempLE);
			sendString(portToHost,outgoing);						
		}
		else
			if(portToDevice==UART_3)
			{
				C_Response_Pending=0;
				sprintf(outgoing, "\r\nC.LE=%d", tempLE);
				sendString(portToHost,outgoing);					
			}
			else
				if(portToDevice==UART_4)
				{
					D_Response_Pending=0;
					sprintf(outgoing, "\r\nD.LE=%d", tempLE);
					sendString(portToHost,outgoing);					
				}
	
	composePrompt();
}

void ox_BE_invalHandler(void)
{
	if(strcmp(myOxCmdTok,INVALID_REPLY)==0)
	{
		A_Response_Pending=0;
		B_Response_Pending=0;
		C_Response_Pending=0;
		D_Response_Pending=0;
		GLOBAL_Response_Pending=0;		
	}
}

void ox_BEq_etHandler(void)
{
	double tempEt=0;

	//Example 24.029 C
	if(tokens[0]!=NULL)
		tempEt = atof(tokens[0]);
	if(portToDevice==UART_1)
		{
			A_Response_Pending=0;
			sprintf(outgoing, "\r\nA.?CT=%3.1f", tempEt);
			sendString(portToHost,outgoing);				
		}
		else
			if(portToDevice==UART_2)
			{
				B_Response_Pending=0;
				sprintf(outgoing, "\r\nB.?CT=%3.1f", tempEt);
				sendString(portToHost,outgoing);						
			}
			else
				if(portToDevice==UART_3)
				{
					C_Response_Pending=0;
					sprintf(outgoing, "\r\nC.?CT=%3.1f", tempEt);
					sendString(portToHost,outgoing);					
				}
				else
					if(portToDevice==UART_4)
					{
						D_Response_Pending=0;
						sprintf(outgoing, "\r\nD.?CT=%3.1f", tempEt);
						sendString(portToHost,outgoing);					
					}
	composePrompt();
}

void ox_BEq_dlfcHandler(void)
{
	//Match myOxCmdTok to the AL table above, return approp messages
	ushort idx=0;
	ushort tempFC=0;

	while(dl_q_Response[idx].resp[0]!=0)
	{
		if(strcmp(myOxCmdTok,dl_q_Response[idx].resp)==0)
			break;
		idx++;
	}
	switch(idx)
	{
		case 0: //Laser Off
			tempFC=1; //Standby
			break;
		case 1: //Setting Temp
			tempFC=2; //Warmup
			break;
		case 2: //Waiting for Stabilization
			tempFC=2; //Warmup
			break;
		case 3: //Starting Up
			tempFC=2; //Warmup
			break;
		case 4: //Laser On
			tempFC=0; //FC=0
			break;
		case 5: //Laser Alarm
			tempFC=17; //Interlock
			break;
		case 6: //Power Lock Lost
			tempFC=512;
			break;
		case 7: //Fan Warning
			tempFC=4096;
			break;
		case 8: //Power Supply Warning
			tempFC=8192;
			break;
		case 9: //Temperature Warning
			tempFC=16384;
			break;
		default:
			tempFC=0;
			break;
	}
	if(portToDevice==UART_1)
		{
			A_Response_Pending=0;
			sprintf(outgoing, "\r\nA.?FC=%d", tempFC);
			sendString(portToHost,outgoing);				
		}
		else
			if(portToDevice==UART_2)
			{
				B_Response_Pending=0;
				sprintf(outgoing, "\r\nB.?FC=%d", tempFC);
				sendString(portToHost,outgoing);						
			}
			else
				if(portToDevice==UART_3)
				{
					C_Response_Pending=0;
					sprintf(outgoing, "\r\nC.?FC=%d", tempFC);
					sendString(portToHost,outgoing);					
				}
				else
					if(portToDevice==UART_4)
					{
						D_Response_Pending=0;
						sprintf(outgoing, "\r\nD.?FC=%d", tempFC);
						sendString(portToHost,outgoing);					
					}
	composePrompt();
	
	
}

void ox_BEq_dlfdHandler(void)
{
	//Return message string as returned from laser
	
	if(portToDevice==UART_1)
		{
			A_Response_Pending=0;
			sprintf(outgoing, "\r\nA.?FD=%s", myOxCmdTok);
			sendString(portToHost,outgoing);				
		}
		else
			if(portToDevice==UART_2)
			{
				B_Response_Pending=0;
				sprintf(outgoing, "\r\nB.?FD=%s", myOxCmdTok);
				sendString(portToHost,outgoing);						
			}
			else
				if(portToDevice==UART_3)
				{
					C_Response_Pending=0;
					sprintf(outgoing, "\r\nC.?FD=%s", myOxCmdTok);
					sendString(portToHost,outgoing);					
				}
				else
					if(portToDevice==UART_4)
					{
						D_Response_Pending=0;
						sprintf(outgoing, "\r\nD.?FD=%s", myOxCmdTok);
						sendString(portToHost,outgoing);					
					}
	composePrompt();
	
	
}

void ox_BEq_btHandler(void)
{
	double tempBt=0;

	//Example 24.029 C
	if(tokens[0]!=NULL)
		tempBt = atof(tokens[0]);

	if(portToDevice==UART_1)
		{
			A_Response_Pending=0;
			sprintf(outgoing, "\r\nA.?OBT=%3.1f", tempBt);
			sendString(portToHost,outgoing);				
		}
		else
			if(portToDevice==UART_2)
			{
				B_Response_Pending=0;
				sprintf(outgoing, "\r\nB.?OBT=%3.1f", tempBt);
				sendString(portToHost,outgoing);						
			}
			else
				if(portToDevice==UART_3)
				{
					C_Response_Pending=0;
					sprintf(outgoing, "\r\nC.?OBT=%3.1f", tempBt);
					sendString(portToHost,outgoing);					
				}
				else
					if(portToDevice==UART_4)
					{
						D_Response_Pending=0;
						sprintf(outgoing, "\r\nD.?OBT=%3.1f", tempBt);
						sendString(portToHost,outgoing);					
					}
	composePrompt();	
}

void ox_BEq_dlHandler(void)
{
	//Match myCmdTok to the DL table above, return approp messages
	ushort idx=0;
	uchar tempLE=0;

	while(dl_q_Response[idx].resp[0]!=0)
	{
		if(strcmp(myOxCmdTok,dl_q_Response[idx].resp)==0)
			break;
		idx++;
	}
	switch(idx)
	{
		case 0: //Laser Off
			tempLE=0;
			break;
		case 1: //Setting Temp
			tempLE=1;
			break;
		case 2: //Waiting for stablization
			tempLE=1;
			break;
		case 3: //Starting Up
			tempLE=1;
			break;
		case 4: //Laser On
			tempLE=1;
			break;
		case 5: //Laser Alarm	
			tempLE=0;	
			break;
		case 6: //Power Lock Lost
			tempLE=0;
			break;
		case 7: //Fan Warning
			tempLE=0;
			break;
		case 8: //Power Supply Warning
			tempLE=0;
			break;
		case 9: //Temp Warning
			tempLE=0;
			break;
		default:
			break;
	}
	if(portToDevice==UART_1)
		{
			A_Response_Pending=0;
			sprintf(outgoing, "\r\nA.?LE=%d", tempLE);
			sendString(portToHost,outgoing);				
		}
		else
			if(portToDevice==UART_2)
			{
				B_Response_Pending=0;
				sprintf(outgoing, "\r\nB.?LE=%d", tempLE);
				sendString(portToHost,outgoing);						
			}
			else
				if(portToDevice==UART_3)
				{
					C_Response_Pending=0;
					sprintf(outgoing, "\r\nC.?LE=%d", tempLE);
					sendString(portToHost,outgoing);					
				}
				else
					if(portToDevice==UART_4)
					{
						D_Response_Pending=0;
						sprintf(outgoing, "\r\nD.?LE=%d", tempLE);
						sendString(portToHost,outgoing);					
					}
	composePrompt();
}

void ox_BEq_ipHandler(void)
{
	double tempPwr=0.0;

	//Example 3.19 mW
	if(tokens[0]!=NULL)
		tempPwr=atof(tokens[0]);
	if(portToDevice==UART_1)
	{
		A_Response_Pending=0;
		sprintf(outgoing, "\r\nA.?LP=%3.2f", tempPwr);
		sendString(portToHost,outgoing);				
	}
	else
		if(portToDevice==UART_2)
		{
			B_Response_Pending=0;
			sprintf(outgoing, "\r\nB.?LP=%3.2f", tempPwr);
			sendString(portToHost,outgoing);						
		}
		else
			if(portToDevice==UART_3)
			{
				C_Response_Pending=0;
				sprintf(outgoing, "\r\nC.?LP=%3.2f", tempPwr);
				sendString(portToHost,outgoing);					
			}
			else
				if(portToDevice==UART_4)
				{
					D_Response_Pending=0;
					sprintf(outgoing, "\r\nD.?LP=%3.2f", tempPwr);
					sendString(portToHost,outgoing);					
				}	
	composePrompt();
}
void ox_BEq_tmHandler(void)
{
	double tempTime=0;

	//Example TIME= 227 hrs, 20 min,
	if(tokens[1]!=NULL)
		tempTime=atof(tokens[1]);
	if(portToDevice==UART_1)
	{
		A_Response_Pending=0;
		sprintf(outgoing, "\r\nA.?LH=%.1f", tempTime);
		sendString(portToHost,outgoing);				
	}
	else
		if(portToDevice==UART_2)
		{
			B_Response_Pending=0;
			sprintf(outgoing, "\r\nB.?LH=%.1f", tempTime);
			sendString(portToHost,outgoing);						
		}
		else
			if(portToDevice==UART_3)
			{
				C_Response_Pending=0;
				sprintf(outgoing, "\r\nC.?LH=%.1f", tempTime);
				sendString(portToHost,outgoing);					
			}
			else
				if(portToDevice==UART_4)
				{
					D_Response_Pending=0;
					sprintf(outgoing, "\r\nD.?LH=%.1f", tempTime);
					sendString(portToHost,outgoing);					
				}	
	composePrompt();
}
void ox_BEq_veHandler(void)
{
	if(tokens[0]!=NULL)
	{
		//Example V3.3.3 SPEC3
		if(portToDevice==UART_1)
		{
			A_Response_Pending=0;
			sprintf(outgoing, "\r\nA.?FV=%s", tokens[0]);
			sendString(portToHost,outgoing);				
		}
		else
			if(portToDevice==UART_2)
			{
				B_Response_Pending=0;
				sprintf(outgoing, "\r\nB.?FV=%s", tokens[0]);
				sendString(portToHost,outgoing);						
			}
			else
				if(portToDevice==UART_3)
				{
					C_Response_Pending=0;
					sprintf(outgoing, "\r\nC.?FV=%s", tokens[0]);
					sendString(portToHost,outgoing);					
				}
				else
					if(portToDevice==UART_4)
					{
						D_Response_Pending=0;
						sprintf(outgoing, "\r\nD.?FV=%s", tokens[0]);
						sendString(portToHost,outgoing);					
					}	
	}
	composePrompt();
}

void resetParser()
{
	if(portToDevice==UART_1)
	{
		A_Response_Pending=0;				
	}
	else
		if(portToDevice==UART_2)
		{
			B_Response_Pending=0;						
		}
		else
			if(portToDevice==UART_3)
			{
				C_Response_Pending=0;					
			}
			else
				if(portToDevice==UART_4)
				{
					D_Response_Pending=0;					
				}
	clearInputBuff(portToDevice);
}


void backEndparserOx(char myPort,char myOriginPort, char *myBuffer, unsigned char myMsgSize)
{
	ushort myCnt;
	char* myTokPtr;

	portToDevice=myOriginPort;
		
	for(myCnt = 0; myCnt < BE_MAX_CMD_SIZE; )	 // clear arrays
	{
		myOxCmdTokCpy[myCnt]=0;
		myOxCmdTokCpy2[myCnt]=0;
 		myOxCmdTok[myCnt++] = 0;
	}

	for(tokIndex=0;tokIndex<10;tokIndex++)
		tokens[tokIndex]=NULL;

	memcpy(myOxCmdTokCpy2,myBuffer,myMsgSize);

	//Remove any line feeds or carriage returns in replies
	myTokPtr=myOxCmdTokCpy2;
	myCnt=0;
	while(*myTokPtr!=NULL && myCnt<myMsgSize)
	{
		if(*myTokPtr!=0x0D && *myTokPtr!=0x0A)
		{
			myOxCmdTok[myCnt++] = *myTokPtr;
		}
		*myTokPtr++;	
	}
	myOxCmdTok[myCnt]=0x00; //Terminate the string

	strupr(myOxCmdTokCpy,myOxCmdTok); //Make a copy in upper case for strtok to modify
	strcpy(myOxCmdTok,myOxCmdTokCpy); //Keep a clean copy for direct use in handlers

	if(strcmp(myOxCmdTok,INVALID_REPLY)==0) //????
	{		
		sprintf(outgoing, "\r\n????");
		sendString(EXT_UART,outgoing);
	}

	if(strcmp(myOxCmdTok,NA)==0) 
	{
		sprintf(outgoing, "\r\nNOT AUTHORIZED");
		sendString(EXT_UART,outgoing);
	}
#if 0		
	for(tokIndex=0;tokIndex<10;tokIndex++)
	{
		tokens[tokIndex]=TEST;
	}
#endif

#if 0

	tokIndex=0;
	myCnt=0;
	myTokPtr=myOxCmdTokCpy;
	while(*myTokPtr!=NULL)
	{
		tokens[tokIndex++]=myTokPtr;
		while(myCnt<myMsgSize)
		{
			if(*myTokPtr==' ')
			{
				*myTokPtr=NULL;			
			}
			myTokPtr++;
			myCnt++;
		}
	}
#endif
	
	tokIndex=0;
	tokens[tokIndex]=strtok(myOxCmdTokCpy,&delim);
  	if(tokens[tokIndex]!=NULL)
	{
   		for(tokIndex=1;tokIndex<10;tokIndex++)
   		{
    		tokens[tokIndex]=strtok(NULL,&delim);
    		if(tokens[tokIndex]==NULL)
			{
      			break;
			}
   		}  	
   	}
	else
	{
		sprintf(outgoing, "\r\nNO TOKENS");
		sendString(EXT_UART,outgoing);
	}	

	//sprintf(outgoing, "\r\n%d*%s*%s*%s*", cmdToMatch[2],myOxCmdTok,tokens[0],tokens[1]);
	//sendString(EXT_UART,outgoing);

	
	switch(cmdToMatch[portToDevice])
	{
		case 0:
			resetParser();
			cmdToMatch[portToDevice]=0;
			return;
		case 1: //Enable Laser Emission
			ox_BE_dlOnHandler();
			break;
		case 2: //Query Laser State, return fault code
			ox_BEq_dlfcHandler(); 
			break;
		case 3: //Query Laser State, return in English
			ox_BEq_dlfdHandler();
			break;
		case 4: //Query Case Temperature
			ox_BEq_btHandler();
			break;
		case 5: //Query Laser Emission
			ox_BEq_dlHandler();
			break;
		case 6: //Query Laser Power
			ox_BEq_ipHandler();
			break;
		case 7: //Query Laser Hours
			ox_BEq_tmHandler();
			break;
		case 8: //Query Firmware Version
			ox_BEq_veHandler();
			break;
		case 9:
			ox_BE_invalHandler();
			break;
		case 10: //Query Controller Electronics Temperature
			ox_BEq_etHandler();
			break;
		case 11: //Disable Laser Emission
			ox_BE_dlOffHandler();
			break;
		case 12: //Detection of lasers			
			resetParser();
			cmdToMatch[portToDevice]=0; //Reset the match parameter
			return;	
			break;
		default:
			composePrompt();
			break;
	}
	cmdToMatch[portToDevice]=0; //Reset the match parameter
	
}

//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
//End OXXIUS Constants, Structs, Functions
//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

int backEndHandler(void)
{
	while(0);
	return 1;	
}	

char* getFcFromPrompt(char *inString)
{
	char inStringCopy[64]={0};
	char matchString[9]={'S','t','r','a','d','u','s',':',0x00};
	char* fcPtr=NULL;
	char* fc=NULL;
			
	strcpy(inStringCopy,inString);
	fcPtr=strstr(inStringCopy,matchString);
	
	if(fcPtr!=NULL)
	{
		fcPtr+=8;	
		fc=fcPtr;
		while(*fcPtr!='>')
		{
			fcPtr++;	
		}
		*fcPtr='\0';
		return fc;
	}
	else
	{
		return NULL;	
	}					
}	

//************************************************************
/*
	char* getFcFromPrompt(char *inString)
	{
		char inStringCopy[64]={0};
		char matchString[9]={'S','t','r','a','d','u','s',':',0x00};
		char* matchPtr;
		char* fcPtr;
		char match=FALSE;
		char* fc=NULL;
			
		strcpy(inStringCopy,inString);
	
		fcPtr=inStringCopy;
		matchPtr=matchString;
		
		while(*fcPtr!=NULL)
		{
			if(*matchPtr==0x00) //Complete "Stradus:" Match
			{
				fc=fcPtr; //Should point at fault code + '>'+ ' ' now
				while(*fcPtr++!='>');
				fcPtr++;
				*fcPtr=NULL;		
				match=TRUE;
				break;
			}
			else
			{
				if(*fcPtr==*matchPtr)
				{
					fcPtr++;
					matchPtr++;
				}					
				else
				{	
					fcPtr++;
				}	
			}					
		}
		
		if(match==TRUE)
		{		
			sprintf(outgoing, "\r\n%s\r\n",*fc);
			sendString(portToHost,outgoing);					
			return fc;			
		}
		else
		{
			sprintf(outgoing, "\r\nNULL FC: %s\r\n",*fc);
			sendString(portToHost,outgoing);
			return NULL;
		}				
	}
	*/

	
//************************************************************
// This parser interprets replies from Stradus Lasers
void backEndparser(char myPort,char myOriginPort, char *myBuffer, unsigned char myMsgSize)
{
	unsigned char indx = 0;
	char *myTmpPtr;
	unsigned char myLength;
	unsigned char myTmpLength;
	unsigned char cmdLength = 0;
	unsigned char myCnt = 0;
	unsigned char cmdMatched = 0;
	char	didWeFindCrKey = FALSE;
	char 	didWeFindEndOfCmdToken = FALSE;
	char* fcStringPtr;
	
	portToDevice=myOriginPort;

	for(myCnt = 0; myCnt < BE_MAX_CMD_SIZE; )	 // clear arrays
	{
		myBETmpBuf[myCnt] = 0;		
 		myCmdTok[myCnt++] = 0;
	}		
	
	//Is it a prompt?
	fcStringPtr=getFcFromPrompt(myBuffer);
	if(fcStringPtr!=NULL)
	{
		char * A_FC;
		char * B_FC;
		char * C_FC;
		char * D_FC;
	
		if(portToDevice==UART_1)
		{
			A_FC=shadowRegs.A_qFC;
			while(*fcStringPtr)
			{
				*A_FC++=*fcStringPtr++;			
			}	
			*A_FC=NULL;							
		}
		else
			if(portToDevice==UART_2)
			{
				B_FC=shadowRegs.B_qFC;
				while(*fcStringPtr)
				{
					*B_FC++=*fcStringPtr++;			
				}
				*B_FC=NULL;	
			}
			else
				if(portToDevice==UART_3)
				{
					C_FC=shadowRegs.C_qFC;
					while(*fcStringPtr)
					{
						*C_FC++=*fcStringPtr++;			
					}
					*C_FC=NULL;						
				}
				else
					if(portToDevice==UART_4)
					{
						D_FC=shadowRegs.D_qFC;
						while(*fcStringPtr)
						{
							*D_FC++=*fcStringPtr++;			
						}
						*D_FC=NULL;						
					}
					return; //Ignore Prompts			
	}	
	
	strcpy(myBETmpBuf,myBuffer);
	strcpy(myCmdTok,myBuffer);

	myTmpPtr = myCmdTok;
	myCnt = 0;
	
	//String format is:  Stradus:0> le=1\r\r\nLE=1\n	

	// truncate the input to just the "command" no spaces, New Line character, equal signs, or arguments
	while(*myTmpPtr	!= NL_KEY && *myTmpPtr != 0)
	{
		if(*myTmpPtr == ' ')
		{
			didWeFindEndOfCmdToken = TRUE;
		 	*myTmpPtr = NULL;		// end of command token.
			break;
		}
		else if(*myTmpPtr == '=')
		{
			didWeFindEndOfCmdToken = TRUE;
		 	*myTmpPtr = NULL;		// end of command token.
			break;
		}
		else
		{
			myTmpPtr++;
		}

		if(myCnt++ > BE_MAX_CMD_SIZE)		// did we exceed the size?
		{
			*myCmdTok = NULL;					// yes, toss input
		 	break;	// error shall be handled in following statements
		}
	}
	
	
	
	//Check for reply from "VORT" command truncated to first word
	if(strcmp(myCmdTok,"Manuf")==0)
	{
		mfrModeResponseHandler();
		return;
	}	

	if(*myTmpPtr == NL_KEY)
	{
		didWeFindCrKey = TRUE;
		*myTmpPtr = NULL;					// yes, remove the Newline character
	}

	if(myCnt > BE_MAX_CMD_SIZE)
	{
		return;	// did not get the "NL"
	}
	else if(didWeFindEndOfCmdToken == TRUE)	// found "=" or " ", equal character or space
	{
		// is there a NL Key entered?
		myTmpPtr++;

		// search for the NL Key
		while(*myTmpPtr	!= NL_KEY && *myTmpPtr != 0)
		{
			myTmpPtr++;

			if(myCnt++ > BE_MAX_CMD_SIZE)		// did we exceed the size?
			{
				*myCmdTok = NULL;					// yes, toss input
			 	break;	// error shall be handled in following statements
			}
		}

		if(*myTmpPtr == NL_KEY)
		{
			didWeFindCrKey = TRUE;
			*myTmpPtr = NULL;					// yes, remove the Newline character
			cmdLength = strlen(myCmdTok);
		}
		else			// did not find NL key!! No reason to parse command
			return;
			
		//Toss any echoes
		myTmpPtr=myCmdTok;
		while(*myTmpPtr++)
		{
			if((*myTmpPtr >= 0x61) && (*myTmpPtr <= 0x7A))
				return;
		}	
			
		
	}
	else if(didWeFindCrKey == FALSE)
		return;	// did not get the "NL"
			
	// loop through all available valid commands looking for a match
	while(VerifyCmds[indx].cmd[0] != 0)	 	
	{
		myLength = strlen(VerifyCmds[indx].cmd);

		if(strcmp(myCmdTok, VerifyCmds[indx].cmd) == 0)	// is this a valid command?	 Note: Could use hashing in the future.
		{
			if(VerifyCmds[indx].maxArgs == 0)		 // Pass Arguments?
			{
				if(myBETmpBuf[myLength] == NL_KEY)		// yes, same length
				{
					myBETmpBuf[myLength] = NULL;		// change the carriage return into a NULL character

					VerifyCmds[indx].cmdHandler();		// no arguments

					cmdMatched = 1;
				}
			}
			else if(VerifyCmds[indx].maxArgs == 1)		// 1 argument expected
			{
				if(myBETmpBuf[myLength + 3] == NL_KEY)
				{
					myBETmpBuf[myLength + 3] = NULL;		// change the carriage return into a NULL character

					argmt1 = (char*) myBETmpBuf;	// yes, 1 argument

					myTmpLength = myLength;

					while(myTmpLength--)	 // move past the command token
						argmt1++;

					while(*argmt1 == ' ')
					{
						argmt1++;	// move passed the space(s)
						myLength++;
					}

					if(*argmt1 == '=')
					{
						argmt1++;	// move passed the equal character '='
						myLength++;
					}

					VerifyCmds[indx].cmdHandler();	// PASS TOKENS!!!

					cmdMatched = 1;
				}
				else
				{
					myTmpPtr = &myBETmpBuf[myLength];
	
					myTmpLength = myLength;
	
					// remove spaces
					while(*myTmpPtr == ' ')
					{
						myTmpPtr++;
						myLength++;
					}
	
					// is there an "=" character"
					if(*myTmpPtr == '=')
					{
						myTmpPtr++;		// move to next character
						myLength++;
	
						// remove spaces
						while(*myTmpPtr == ' ')
						{
							myTmpPtr++;
							myLength++;
						}
	
						// get argument/token
						argmt1 = myTmpPtr;	// yes, 1 argument
	
						// find the "NL"
						while(*myTmpPtr != NL_KEY)
						{
							myTmpPtr++;
							myLength++;
						}
	
						*myTmpPtr = 0;	// replace the "NL" with a NULL
						myLength++;
	
						VerifyCmds[indx].cmdHandler();	// PASS TOKENS!!!
	
						cmdMatched = 1;
					}	
				}					
			}
			if(cmdMatched)
				break;	
		}
		indx++;
	}
	
	//Command not found in table. Is it still a valid command?
	if(cmdMatched==0)
	{	
		if(isManufModeActive==TRUE)						 		
			BEgeneralBackHandler(cmdLength);
		else
			emptyResponseHandler();			
	}	
}

//Command Handler Functions**********************************************************

void clearGlobalResponseBuffers()
{
	int i=0;
	for(i=0;i<64;i++)
	{
		globalResponseBuffA[i]=NULL;
		globalResponseBuffB[i]=NULL;
		globalResponseBuffC[i]=NULL;
		globalResponseBuffD[i]=NULL;		
	}					
}	

void globalResponseHandler()
{
	if(portToDevice==UART_1)
	{
		A_Response_Pending=0;
		sprintf((char*)globalResponseBuffA, "%s=%s", myCmdTok,argmt1);					
	}
	else
		if(portToDevice==UART_2)
		{
			B_Response_Pending=0;
			sprintf((char*)globalResponseBuffB, "%s=%s", myCmdTok,argmt1);						
		}
		else
			if(portToDevice==UART_3)
			{
				C_Response_Pending=0;
				sprintf((char*)globalResponseBuffC, "%s=%s", myCmdTok,argmt1);				
			}
			else
				if(portToDevice==UART_4)
				{
					D_Response_Pending=0;
					sprintf((char*)globalResponseBuffD, "%s=%s", myCmdTok,argmt1);				
				}

	if(GLOBAL_Response_Pending&&!A_Response_Pending&&!B_Response_Pending&&!C_Response_Pending&&!D_Response_Pending)
	{
		GLOBAL_Response_Pending=0;
		sprintf(outgoing, "\r\nA.%s\r\nB.%s\r\nC.%s\r\nD.%s", globalResponseBuffA,globalResponseBuffB,globalResponseBuffC,globalResponseBuffD);		
		clearGlobalResponseBuffers();

		sendString(portToHost,outgoing);
		composePrompt();	
	}		
}

void emptyResponseHandler()
{
	if(portToDevice==UART_1)
	{
		if(A_Response_Pending==1)		
			A_Response_Pending=0;
		else
			return;					
	}
	else
		if(portToDevice==UART_2)
		{
			if(B_Response_Pending==1)
				B_Response_Pending=0;
			else
				return;						
		}
		else
			if(portToDevice==UART_3)
			{
				if(C_Response_Pending==1)
					C_Response_Pending=0;
				else
					return;				
			}
			else
				if(portToDevice==UART_4)
				{
					if(D_Response_Pending==1)	
						D_Response_Pending=0;					
					else
						return;					
				}
	composePrompt();	
}	

void individualResponseHandler()
{	
	if(portToDevice==UART_1)
	{
		if(A_Response_Pending==1)
		{		
			sprintf(outgoing, "\r\nA.%s=%s", myCmdTok,argmt1);
			sendString(portToHost,outgoing);
			A_Response_Pending=0;
		}
		else
			return;					
	}
	else
		if(portToDevice==UART_2)
		{
			if(B_Response_Pending==1)
			{
				sprintf(outgoing, "\r\nB.%s=%s", myCmdTok,argmt1);
				sendString(portToHost,outgoing);
				B_Response_Pending=0;
			}
			else
				return;						
		}
		else
			if(portToDevice==UART_3)
			{
				if(C_Response_Pending==1)
				{
					sprintf(outgoing, "\r\nC.%s=%s", myCmdTok,argmt1);
					sendString(portToHost,outgoing);
					C_Response_Pending=0;
				}
				else
					return;				
			}
			else
				if(portToDevice==UART_4)
				{
					if(D_Response_Pending==1)
					{	
						sprintf(outgoing, "\r\nD.%s=%s", myCmdTok,argmt1);
						sendString(portToHost,outgoing);
						D_Response_Pending=0;
					}						
					else
						return;
					
				}	
	composePrompt();
}

void mfrModeResponseHandler()
{
	if(portToDevice==UART_1)
	{
		A_Response_Pending=0;
		sprintf(outgoing, "\r\nA.MfrMode=1");
		sendString(portToHost,outgoing);				
	}
	else
		if(portToDevice==UART_2)
		{
			B_Response_Pending=0;
			sprintf(outgoing, "\r\nB.MfrMode=1");
			sendString(portToHost,outgoing);						
		}
		else
			if(portToDevice==UART_3)
			{
				C_Response_Pending=0;
				sprintf(outgoing, "\r\nC.MfrMode=1");
				sendString(portToHost,outgoing);					
			}
			else
				if(portToDevice==UART_4)
				{
					D_Response_Pending=0;
					sprintf(outgoing, "\r\nD.MfrMode=1");
					sendString(portToHost,outgoing);					
				}
	composePrompt();	
}				

void BEgeneralBackHandler(unsigned char myLength)
{	
	unsigned char myTempLength=0;
	char* myTempPtr;
		
	//If only a newline-prompt or just newline are returned
	if(myBETmpBuf[0] == NL_KEY)
	{
		myBETmpBuf[0] = NULL;		// change the newline into a NULL character
		
		if(portToDevice==UART_1)
		{
			A_Response_Pending=0;
		}
		if(portToDevice==UART_2)
		{
			B_Response_Pending=0;
		}
		if(portToDevice==UART_3)
		{
			C_Response_Pending=0;
		}
		if(portToDevice==UART_4)
		{
			D_Response_Pending=0;
		}
		composePrompt();
		return;
	}

	myTempPtr = &myBETmpBuf[myLength];

	myTempLength = myLength;

	// remove spaces
	while(*myTempPtr == ' ')
	{
		myTempPtr++;
		myLength++;
	}

	// is there an "=" character"
	if(*myTempPtr == '=')
	{
		myTempPtr++;		// move to next character
		myLength++;

		// remove spaces
		while(*myTempPtr == ' ')
		{
			myTempPtr++;
			myLength++;
		}

		// get argument/token
		argmt1 = myTempPtr;	// yes, 1 argument
		
		//Eliminate any Carriage Returns or Newlines
		while(*myTempPtr != 0)
		{
			if(*myTempPtr==CR_KEY || *myTempPtr==NL_KEY)
				*myTempPtr=0;
			myTempPtr++;
			myLength++;	
		}	
	}										
	
	if(portToDevice==UART_1)
	{
		if(A_Response_Pending==1)
		{		
			sprintf(outgoing, "\r\nA.%s=%s", myCmdTok,argmt1);
			sendString(portToHost,outgoing);
			A_Response_Pending=0;
		}				
	}
	else
		if(portToDevice==UART_2)
		{
			if(B_Response_Pending==1)
			{
				sprintf(outgoing, "\r\nB.%s=%s", myCmdTok,argmt1);
				sendString(portToHost,outgoing);
				B_Response_Pending=0;
			}						
		}
		else
			if(portToDevice==UART_3)
			{
				if(C_Response_Pending==1)
				{
					sprintf(outgoing, "\r\nC.%s=%s", myCmdTok,argmt1);
					sendString(portToHost,outgoing);
					C_Response_Pending=0;
				}				
			}
			else
				if(portToDevice==UART_4)
				{
					if(D_Response_Pending==1)
					{	
						sprintf(outgoing, "\r\nD.%s=%s", myCmdTok,argmt1);
						sendString(portToHost,outgoing);
						D_Response_Pending=0;
					}										
				}	
	composePrompt();		
}	


//Individual
void BEcHandler(void)
{
	individualResponseHandler();	
}

//Individual
void BEdelayHandler(void)
{
	individualResponseHandler();	
}

//Individual
void BEepcHandler(void)
{
	individualResponseHandler();
}

//Individual
void BEleHandler(void)
{
	individualResponseHandler();
}
	
//Individual
void BElpHandler(void)
{
	individualResponseHandler();			
}

void BEpromptHandler(void)
{
	if(configuring==TRUE)
	{
		//Do not send any responses out serial ports during power-up testing
		if(portToDevice==UART_1)
		{
			A_Response_Pending=0;				
		}
		else
			if(portToDevice==UART_2)
			{
				B_Response_Pending=0;						
			}
			else
				if(portToDevice==UART_3)
				{
					C_Response_Pending=0;					
				}
				else
					if(portToDevice==UART_4)
					{
						D_Response_Pending=0;					
					}		
	}
	else
	{
		individualResponseHandler();
	}
}

//Individual
void BEtecHandler(void)
{
	individualResponseHandler();
}
//Individual
void BEvobtsHandler(void)
{
	individualResponseHandler();
}

//Individual Pulse Power Mode On/Off
void BEpulHandler(void)
{
	individualResponseHandler();			
}

//Individual Set Pulse Power
void BEppHandler(void)
{
	individualResponseHandler();			
}

//Obtained from theChosenOne
void BEq_bptHandler(void)
{
	if(portToDevice==UART_1)
	{
		A_Response_Pending=0;
	}
	if(portToDevice==UART_2)
	{
		B_Response_Pending=0;
	}
	if(portToDevice==UART_3)
	{
		C_Response_Pending=0;
	}
	if(portToDevice==UART_4)
	{
		D_Response_Pending=0;
	}
	sprintf(outgoing, "\r\n%s=%s", myCmdTok,argmt1);
	sendString(portToHost,outgoing);								

	composePrompt();	
}

//Individual
void BEq_cHandler(void)
{
	individualResponseHandler();
}

//Individual
void BEq_ccHandler(void)
{
	individualResponseHandler();
}

//Individual
void BEq_delayHandler(void)
{
	individualResponseHandler();	
}
//Individual
void BEq_epcHandler(void)
{
	individualResponseHandler();
}
//Individual
void BEq_fcHandler(void)
{
	char * A_FC;
	char * B_FC;
	char * C_FC;
	char * D_FC;
	
	if(portToDevice==UART_1)
	{
		A_Response_Pending=0;
		A_FC=shadowRegs.A_qFC;
		while(*argmt1)
		{
			*A_FC++=*argmt1++;			
		}	
		*A_FC=NULL;
		argmt1=shadowRegs.A_qFC;
		sprintf(outgoing, "\r\nA.%s=%s", myCmdTok,argmt1);
		sendString(portToHost,outgoing);				
	}
	else
		if(portToDevice==UART_2)
		{
			B_Response_Pending=0;
			B_FC=shadowRegs.B_qFC;
			while(*argmt1)
			{
				*B_FC++=*argmt1++;			
			}
			*B_FC=NULL;	
			argmt1=shadowRegs.B_qFC;
			sprintf(outgoing, "\r\nB.%s=%s", myCmdTok,argmt1);
			sendString(portToHost,outgoing);		
			
		}
		else
			if(portToDevice==UART_3)
			{
				C_Response_Pending=0;
				C_FC=shadowRegs.C_qFC;
				while(*argmt1)
				{
					*C_FC++=*argmt1++;			
				}
				*C_FC=NULL;	
				argmt1=shadowRegs.C_qFC;
				sprintf(outgoing, "\r\nC.%s=%s", myCmdTok,argmt1);
				sendString(portToHost,outgoing);					
			}
			else
				if(portToDevice==UART_4)
				{
					D_Response_Pending=0;
					D_FC=shadowRegs.D_qFC;
					while(*argmt1)
					{
						*D_FC++=*argmt1++;			
					}
					*D_FC=NULL;	
					argmt1=shadowRegs.D_qFC;
					sprintf(outgoing, "\r\nD.%s=%s", myCmdTok,argmt1);
					sendString(portToHost,outgoing);					
				}
	composePrompt();						
}
//Individual
void BEq_fdHandler(void)
{
	individualResponseHandler();	

}
//Individual
void BEq_fpHandler(void)
{	
	individualResponseHandler();
}
//Individual
void BEq_fvHandler(void)
{
	individualResponseHandler();	
}

//Individual
void BEq_ilHandler(void)
{
	if(portToDevice==UART_1)
	{
		A_Response_Pending=0;
	}
	if(portToDevice==UART_2)
	{
		B_Response_Pending=0;
	}
	if(portToDevice==UART_3)
	{
		C_Response_Pending=0;
	}
	if(portToDevice==UART_4)
	{
		D_Response_Pending=0;
	}
	sprintf(outgoing, "\r\n%s=%s", myCmdTok,argmt1);
	sendString(portToHost,outgoing);								

	composePrompt();			
}
//Individual
void BEq_lcHandler(void)
{
	individualResponseHandler();
}
//Individual
void BEq_leHandler(void)
{
	individualResponseHandler();
}
//Individual
void BEq_lhHandler(void)
{
	individualResponseHandler();
}
//Individual
void BEq_liHandler(void)
{
	individualResponseHandler();
}
//Individual
void BEq_lpHandler(void)
{
	individualResponseHandler();
}
//Individual
void BEq_lpsHandler(void)
{
	individualResponseHandler();
}
//Individual
void BEq_lwHandler(void)
{
	individualResponseHandler();
}
//Individual
void BEq_maxpHandler(void)
{
	individualResponseHandler();
}
//Individual
void BEq_obtHandler(void)
{
	individualResponseHandler();	
}
//Individual
void BEq_obtsHandler(void)
{
	individualResponseHandler();			
}
//Individual
void BEq_rpHandler(void)
{
	individualResponseHandler();	
}
//Individual
void BEq_tecHandler(void)
{
	individualResponseHandler();		
}

//Individual
void BEq_ppHandler(void)
{
	individualResponseHandler();			
}

//Individual
void BEq_pulHandler(void)
{
	individualResponseHandler();			
}

//Individual
void BEq_op1Handler(void)
{
	individualResponseHandler();			
}

//Individual
void BEq_op2Handler(void)
{
	individualResponseHandler();			
}
//Individual
void BEq_op3Handler(void)
{
	individualResponseHandler();			
}
//Individual
void BEq_op4Handler(void)
{
	individualResponseHandler();
}



