//circular_queue.c

#include "circular_queue.h"

Q_CONFIG qHandle[5];

// globals
#if 0
unsigned char inQTmp[MAX_Q_BUF_SIZE];
unsigned char outQTmp[MAX_Q_BUF_SIZE];
#endif


// initialize all of the queues
void qInit(void)
{
	unsigned short int myCnt;	
	unsigned char myIndx;

	// initialize the head and tail pointers to the first element of the queue, for all queues
	for(myIndx = 0; myIndx < END_OF_QUEUES; myIndx++)
	{
		qHandle[myIndx].qPtrHead[IN_Q] = &qHandle[myIndx].queue[IN_Q][0];
		qHandle[myIndx].qPtrTail[IN_Q] = &qHandle[myIndx].queue[IN_Q][0];
		
		qHandle[myIndx].qPtrHead[OUT_Q] = &qHandle[myIndx].queue[OUT_Q][0];
		qHandle[myIndx].qPtrTail[OUT_Q] = &qHandle[myIndx].queue[OUT_Q][0];
		
		// initialize ALL of the in/out queues to zero
		for(myCnt = 0; myCnt < MAX_Q_BUF_SIZE; myCnt++)
		{
			qHandle[myIndx].queue[IN_Q][myCnt] = 0;	
			qHandle[myIndx].queue[OUT_Q][myCnt] = 0;		
		}		
	}				
}

// Reset a specific queue: head and tail, and queue data buffer 
void qReset(Q_STREAM myStream, Q_TYPE myType)
{
	unsigned char myCnt;
		
	//VICIntEnClr = 1 << 6;       /* Disable UART0 Interrupt */

	// initialize the head and tail pointers to the first element of the queue, for this specific queue
	qHandle[myStream].qPtrHead[myType] = &qHandle[myStream].queue[myType][0];
	qHandle[myStream].qPtrTail[myType] = &qHandle[myStream].queue[myType][0];
		
	// initialize ALL of the in/out queues to zero
	for(myCnt = 0; myCnt < MAX_Q_BUF_SIZE; myCnt++)	// init queue to zero
		qHandle[myStream].queue[myType][myCnt] = 0;	

  //VICIntEnable = 1 << 6;       /* Enable UART0 Interrupt */	May need to use this								
}

// Store a bytes to the specified queue if there is room in the queue
void writeQ(Q_STREAM myQ, Q_TYPE myQType, unsigned char *myData, unsigned char mySize)
{	
	if(mySize >= MAX_Q_BUF_SIZE)	// is the size requested bigger then the max buffer size?
	{
		return;	
	}

	while(mySize--)	// save the data, based on size
	{
		// is there any free room in Q? The head pointer must not be 1 byte behind the tail pointer
		// first check the wrap condition 
		if(qHandle[myQ].qPtrHead[myQType] == &qHandle[myQ].queue[myQType][MAX_Q_BUF_SIZE - 1])	// is this the last position in Q?
		{
			if(qHandle[myQ].qPtrTail[myQType] == &qHandle[myQ].queue[myQType][0])	// pointers shall collide!!! Queue Full!!
			{
				break;
			}
			else
			{
				*qHandle[myQ].qPtrHead[myQType] = *myData++;	// save the data
				qHandle[myQ].qPtrHead[myQType] = &qHandle[myQ].queue[myQType][0];		// wrap to the first start of the queue						
			}	
		}
		else
		{
			if(qHandle[myQ].qPtrHead[myQType] == (qHandle[myQ].qPtrTail[myQType] - 1)) 	// pointers shall collide!!! Queue Full!!
			{
				break;
			}
			else
			{
				*qHandle[myQ].qPtrHead[myQType] = *myData++;	// save the data
				qHandle[myQ].qPtrHead[myQType]++;		// go to the next free element of the queue						
			}			
		}
	}
}


// read bytes from the specified queue if there is data in the queue, This does NOT Flush the data from  the queue
void readQ(Q_STREAM myQ, Q_TYPE myQType, unsigned char *myData, unsigned char mySize)
{
	unsigned char *myCurQHandle;

	myCurQHandle = qHandle[myQ].qPtrTail[myQType];	// save the "current" address
	
	if(mySize >= MAX_Q_BUF_SIZE)	// is the size requested bigger then the max buffer size?
	{
		return;	
	}
	
	while(mySize--)
	{
		// is queue empty? The tail pointer must not be equal to the head pointer
		if(qHandle[myQ].qPtrTail[myQType] == qHandle[myQ].qPtrHead[myQType])
		{
			//qErrorLog(myQ, myQType, Q_READ);	// yes!!! Queue empty
			qHandle[myQ].qPtrTail[myQType] = myCurQHandle;	 // return to the "current" address
			break;
		}
		else
		{
			*myData++ = *qHandle[myQ].qPtrTail[myQType];	// get the data
			
			// make sure the tail is not about to wrap!!
			if(qHandle[myQ].qPtrTail[myQType] == &qHandle[myQ].queue[myQType][MAX_Q_BUF_SIZE - 1]) 	// is this the last position in Q?
				qHandle[myQ].qPtrTail[myQType] = &qHandle[myQ].queue[myQType][0];		// yes, go to start of queue
			else
				qHandle[myQ].qPtrTail[myQType]++;
		}	
	}
	
	qHandle[myQ].qPtrTail[myQType] = myCurQHandle;	 // return to the "current" address	
}

// flush bytes from the specified queue if there is data in the queue
void flushQ(Q_STREAM myQ, Q_TYPE myQType, unsigned char mySize)
{
	unsigned char *myCurQHandle;

	myCurQHandle = qHandle[myQ].qPtrTail[myQType];	// save the "current" address
	
	if(mySize >= MAX_Q_BUF_SIZE)	// is the size requested bigger then the max buffer size?
	{
		return;	
	}
	
	while(mySize--)
	{
		// is queue empty? The tail pointer must not be equal to the head pointer
		if(qHandle[myQ].qPtrTail[myQType] == qHandle[myQ].qPtrHead[myQType])
		{
			//qErrorLog(myQ, myQType, Q_READ);	// yes!!! Queue empty
			qHandle[myQ].qPtrTail[myQType] = myCurQHandle;	 // return to the "current" address
			break;
		}
		else
		{	
			// make sure the tail is not about to wrap!!
			if(qHandle[myQ].qPtrTail[myQType] == &qHandle[myQ].queue[myQType][MAX_Q_BUF_SIZE - 1]) 	// is this the last position in Q?
				qHandle[myQ].qPtrTail[myQType] = &qHandle[myQ].queue[myQType][0];		// yes, go to start of queue
			else
				qHandle[myQ].qPtrTail[myQType]++;
		}	
	}	
}


// the number of bytes that have been written and can be read from the queue
unsigned char qFullIndicator(Q_STREAM myQ, Q_TYPE myQType)
{
	unsigned char myVal;
	
	if((unsigned char *) qHandle[myQ].qPtrHead[myQType] > (unsigned char *) qHandle[myQ].qPtrTail[myQType])
	{
		myVal =  (qHandle[myQ].qPtrHead[myQType] - qHandle[myQ].qPtrTail[myQType]);
	}
	else if((unsigned char *) qHandle[myQ].qPtrHead[myQType] == (unsigned char *) qHandle[myQ].qPtrTail[myQType])
	{

		myVal = 0;
	}
	else
	{		
		myVal =  &qHandle[myQ].queue[myQType][MAX_Q_BUF_SIZE] -  qHandle[myQ].qPtrTail[myQType];
		myVal += ( qHandle[myQ].qPtrHead[myQType] - &qHandle[myQ].queue[myQType][0]); // calculate the head proximity from the start of the queue				
	}

	return(myVal);	
}

// the number of bytes that have been written and can be read from the queue
unsigned char qEmptyIndicator(Q_STREAM myQ, Q_TYPE myQType)
{
	unsigned char myFullSize;	
	myFullSize = qFullIndicator(myQ, myQType);	
	return(MAX_Q_BUF_SIZE - myFullSize);
}


#if 0
int diagTestQueue(void)
{
	unsigned char myCnt;	
	
	qInit();
	
	for(myCnt = 0; myCnt < MAX_Q_BUF_SIZE; myCnt++)
	{
		inQTmp[myCnt] = 0;
		outQTmp[myCnt] = 0;		
	}
	
	// test the queue full and empty indicator	
	myCnt = qFullIndicator(USB_Q, IN_Q);
	myCnt = qEmptyIndicator(USB_Q, IN_Q);
	
	writeQ(USB_Q, IN_Q, inQTmp, 0x3F);
	
	myCnt = qFullIndicator(USB_Q, IN_Q);
	myCnt = qEmptyIndicator(USB_Q, IN_Q);
	
	writeQ(USB_Q, IN_Q, inQTmp, 0x1);
	
	myCnt = qFullIndicator(USB_Q, IN_Q);
	myCnt = qEmptyIndicator(USB_Q, IN_Q);
	
	writeQ(USB_Q, IN_Q, inQTmp, 0x1);
	myCnt = qFullIndicator(USB_Q, IN_Q);
	myCnt = qEmptyIndicator(USB_Q, IN_Q);
	
	
	// debug test" writes to queue to make queue full
	for(myCnt = 0; myCnt < MAX_Q_BUF_SIZE; myCnt++)
		inQTmp[myCnt] = myCnt;
		
	writeQ(USB_Q, IN_Q, inQTmp, 0xF0);
	writeQ(USB_Q, IN_Q, &inQTmp[0x20], 0x10);	

	
	// debug test roll over (wrap around)
	for(myCnt = 0; myCnt < MAX_Q_BUF_SIZE; myCnt++)
		inQTmp[myCnt] = myCnt;
		
	writeQ(USB_Q, IN_Q, inQTmp, 0xF3);
	readQ(USB_Q, IN_Q, outQTmp, 0xF3);
	writeQ(USB_Q, IN_Q, &inQTmp[0x20], 0x10);
	readQ(USB_Q, IN_Q, outQTmp, 0x10);
	return 1;
}
#endif



