//eeprom.c
#include "system.h"
#include "i2c.h"
#include "eeprom.h"
#include "uart.h"
#include "config.h"
#include "p24fj256gb106.h"
#include "string.h"

unsigned char isEepromWorking;
char serialOut[32];

int checkEEPROM(void);

void initEEPROM(void)
{
	if(checkEEPROM() == 0)
	{
		isEepromWorking = 1;
		//sprintf(serialOut,"\r\n***EEPROM TEST PASSED***\r\n");				   	
		//sendString(EXT_UART,serialOut);
	}	
	else
	{
		isEepromWorking = 0;
		//sprintf(serialOut,"\r\n***EEPROM ERROR***\r\n");				   	
		//sendString(EXT_UART,serialOut);  
	}	
}

void eepromRead(unsigned short myAddr ,unsigned char* bytesToRead,unsigned char numBytes)
{
	unsigned int count;
	count = HDByteReadI2C(I2C_ADDR_EEPROM, (myAddr & 0xFF00) >> 8, myAddr, bytesToRead, numBytes);	
}

void eepromWrite(unsigned short myAddr ,unsigned char* bytesToWrite,unsigned char numBytes)
{
	unsigned int count;
	count = HDByteWriteI2C(I2C_ADDR_EEPROM, (myAddr & 0xFF00) >> 8, myAddr, bytesToWrite,numBytes);
}
	
//Check EEPROM functionality by writing and reading some data
int checkEEPROM()
{
	unsigned char diagMSB=0xAB;
	unsigned char diagLSB=0xCD;
	unsigned char dataMSB=0x00;
	unsigned char dataLSB=0x00;
		
	//Read high byte of diagnostic contents from EEPROM
	eepromRead(EE_DIAG_BYTES,&dataMSB,1);
	
	//Read low byte of diagnostic contents from EEPROM
	eepromRead(EE_DIAG_BYTES+1,&dataLSB,1);
		
	//If high byte read does not match diagMSB then write and read location (EEPROM may be new or corrupt)
	if(dataMSB != diagMSB)
	{
		eepromWrite(EE_DIAG_BYTES,&diagMSB,1);
		
		eepromRead(EE_DIAG_BYTES,&dataMSB,1);	
	}	
	
	//If low byte read does not match diagLSB then write and read location (EEPROM may be new or corrupt)
	if(dataLSB != diagLSB)
	{
		eepromWrite(EE_DIAG_BYTES+1,&diagLSB,1);
		
		eepromRead(EE_DIAG_BYTES+1,&dataLSB,1);	
	}	
		
	//If high and low bytes from EEPROM reads do not match diagMSB and diagLSB respectively then EEPROM test fails
	//Two bytes to account for a possible transmission error
	if(dataMSB != diagMSB && dataLSB != diagLSB)
	{
		//EEPROM Test FAILS!!
		return -1;		
	}			
	return 0; //Pass
}	


