#include "p24FJ256GB106.h"
#include "i2c.h"
#include "extUART.h"
#include <string.h>
#include "uart.h"
#include "timers.h"

volatile unsigned char extUARTIntOccurred=0x00;
unsigned char bytesFromFifo[64];



void extUARTInit()
{
	unsigned char byteToWrite=0x00;

	delayMs(3);

	//Initialize Line Control Register
	//Enable Special Register Set (Set LCR[7]=1) to allow divisor change that follows
	//Set 8 bit word, 1 Start, 1 Stop, No Parity
	byteToWrite=0x83;
	extUARTWrite(LCR,&byteToWrite,1);
	//Set Baud Rate Generator Divisor Latch for 19200 bps with LCR[7]=1
	//Divisor = 1843200 / (16 * 19200) = 6	
	delayMs(3);

	//Can only access DLH and DLL only when LCR[7]=1 which should be true at this point.
	//Set DLH (MSB) 0x00
	byteToWrite=0x00;
	extUARTWrite(DLH,&byteToWrite,1);

	delayMs(3);
	//Set DLL (LSB) 0x06
	byteToWrite=0x06;
	extUARTWrite(DLL,&byteToWrite,1);

	delayMs(3);

	byteToWrite=0x03;
	extUARTWrite(LCR,&byteToWrite,1); //Disable access to baud rate divisor

	delayMs(3);

	byteToWrite=0x07; //64 char RX FIFO,FIFOs Enabled (Does not set TX FIFO trigger level since EFR[4] remains 0)
	extUARTWrite(FCR,&byteToWrite,1);

#if 0
	//Interrupt line intended but not used since FIFO has a minimum interrupt trigger level of 4 bytes 
	//and typical RS-232 terminal with echo to user requires a 1 byte trigger level.

	//Enable INT1 (RD8) Interrupt 
    INTCON2bits.INT1EP=1; 	//Falling edge detection
	IFS1bits.INT1IF=0; //Reset INT1 Interrupt Flag 
	IEC1bits.INT1IE=1; //Enable INT1 Interrupt

	delayMs(3);                                             
	
	byteToWrite=0x01; //Set IER[0]=1, Enable RX Holding Register Interrupt
	extUARTWrite(IER,&byteToWrite,1);
#endif

}

unsigned char getExtUARTStatus()
{
	unsigned char status;

	extUARTRead(LSR,&status,1);
	return status;
}



//INT1 Interrupt Handler
void __attribute__((interrupt, no_auto_psv)) _INT1Interrupt( void )
{
	extUARTIntOccurred=0x01;
	IFS1bits.INT1IF=0; //Reset INT1 Interrupt Flag 
}

void getBytesFromUART()
{
	int k;
	unsigned char myByte;

	extUARTRead(RXLVL,&myByte,1);

	if(myByte>0)
	{
		//Get bytes from external UART and load into circular buffer
		extUARTRead(RHR,bytesFromFifo,myByte);
			
		for(k=0;k<64;k++)
			if(bytesFromFifo[k]!=0x00)
				extUARTHdlr(bytesFromFifo[k]);
			
		memset(bytesFromFifo,0x00,64);
	}
}

unsigned char getINTSrcExtUART() //Call this from main loop after polling for interrupt flag
{
	unsigned char sourceOfINT=0;
	
	extUARTRead(IIR,&sourceOfINT,1);
	if(sourceOfINT & 0x04) //RX Holding Register Interrupt
	{
		return 1;
	}
	else
		return 0; //No cause, Error!
}


void extUARTRead(unsigned char myAddr ,unsigned char* bytesToRead,unsigned char numBytes)
{
	unsigned int count;

	//UART internal subAddress: [6:3] is Internal Reg address, bits 7 and 0 are unused, bits 1 and 2 are fixed at 0		
	count = LDByteReadI2C(I2C_ADDR_EXT_UART, myAddr << 3, bytesToRead, numBytes);
}

void extUARTWrite(unsigned char myAddr ,unsigned char* bytesToWrite,unsigned char numBytes)
{
	unsigned int count;
	
	//while(numBytes--)
	//{
		//UART internal subAddress: [6:3] is Internal Reg address, bits 7 and 0 are unused, bits 1 and 2 are fixed at 0	
		count = LDByteWriteI2C(I2C_ADDR_EXT_UART, myAddr <<3, *bytesToWrite++);
	//}
}



