/********************************************************************
 FileName:		main.c
 Dependencies:	See INCLUDES section
 Complier:  	Microchip C30 (for PIC24)
 Company:		Microchip Technology, Inc.

 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the Company) for its PIC Microcontroller is intended and
 supplied to you, the Companys customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
Vortran Stradus MultiLaser Firmware - main.c
Revision History:
	V1.0.0d Initial Release TBD
	V1.0.1d
	!!!!!!!!!!FIX isSystemReady() in ProcessFRONT_SERIAL_EVENTS(USB_PORT)
	V1.0.2 Production Release
	V1.0.3 Added ability to enter manufacturer mode (vort) and to
		set individual laser hours through VersaLase interface (lh=) 
	V1.0.4 When in manufacturing mode, added the ability for 
		Stradus commands following the ?X and X=# formats to be forwarded 
		to the Stradus lasers and their associated responses to be passed 
		back to the sender. This includes manufacturer mode and user mode 
		commands and queries that follow the specified format.

Notes: A single NOP instruction translates to about 1us of delay when
	calibrated with an I/O pin on a scope
	  
*******************************************************************/

/** INCLUDES ***************************************************/
#include "Compiler.h"
#include "HardwareProfile.h"
#include "GenericTypeDefs.h"
#include "USB/usb_device.h"
#include "USB/usb.h"
#include "USB/usb_function_generic.h"
#include "USB/usbvendreq.h"
#include "usb_config.h"
#include "uart.h"
#include "init.h"
#include "parser.h"
#include "system.h"
#include "circular_queue.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "config.h"
#include "timers.h"
#include "extUART.h"
#include "i2c.h"
#include "eeprom.h"

#define U1_LINE_BUFF_SIZE 96
#define U2_LINE_BUFF_SIZE 96
#define U3_LINE_BUFF_SIZE 96
#define U4_LINE_BUFF_SIZE 96
#define UEXT_LINE_BUFF_SIZE 96

#define IDLE 0x00
#define WAIT_PWR_UP 0x01
#define BEGIN_COMM_CHECK 0x02
#define WAIT_COMM_CHECK 0x03
#define READY_FOR_INPUT 0x04
#define NORMAL 0x05
#define CHECK_LASERS 0x06

extern unsigned char getINTSrcExtUART();
extern void parser(char myPort, char *myBuf, Q_STREAM stream, unsigned char myMsgSize);


void processFrontSerialEvents(unsigned char myPort);
void processBackSerialEvents(unsigned char myPort);
void setLaserAsDetected(unsigned char laser);
void setLaserAsUndetected(unsigned char laser);
unsigned char isSystemReady();
void systemProcess(void);
void powerUpDelay(void);
void clearInputBuffs(void);

//*************************************************************
char k1,k2,k3,k4,kExt;
char lineBuff1[U1_LINE_BUFF_SIZE]; //Line Buffer holds received string being parsed
int lineBuffIndex1=0;
char lineBuff2[U2_LINE_BUFF_SIZE]; //Line Buffer holds received string being parsed
int lineBuffIndex2=0;
char lineBuff3[U3_LINE_BUFF_SIZE]; //Line Buffer holds received string being parsed
int lineBuffIndex3=0;
char lineBuff4[U4_LINE_BUFF_SIZE]; //Line Buffer holds received string being parsed
int lineBuffIndex4=0;
char lineBuffExt[UEXT_LINE_BUFF_SIZE];
int lineBuffIndexExt=0;
char serOutBuf[UEXT_LINE_BUFF_SIZE];

extern unsigned char U1RX_Counter,U2RX_Counter,U3RX_Counter,U4RX_Counter, UExtRX_Counter;
extern unsigned char U1TX_Counter,U2TX_Counter,U3TX_Counter,U4TX_Counter, UExtTX_Counter;
extern unsigned char myUARTExtIn[MAX_SERIAL_BUF_SIZE];
extern unsigned char myUARTExtPos;
extern volatile unsigned char extUARTIntOccurred;
extern VOLATILE_REGISTERS shadowRegs;
extern void sendString (unsigned char myPort, char* myBuf);
extern unsigned char isEepromWorking;
void processSerialEvents(unsigned char myPort);
char myStringExt[64];
char myString1[64];
char myString2[64];
char myString3[64];
char myString4[64];
extern unsigned char cmdToMatch[];

unsigned char extUARTStatus;

unsigned long int myCounts=0;
unsigned long int myPrevCounts=0;
unsigned long int mySysCounts=0;
unsigned long int myPrevSysCounts=0;
unsigned long int sysStateCounter=0;

unsigned char A_Response_Pending=0;
unsigned char B_Response_Pending=0;
unsigned char C_Response_Pending=0;
unsigned char D_Response_Pending=0;
unsigned char GLOBAL_Response_Pending=0;
unsigned char configuring;
unsigned char ON_OFF_FLAG=0;
unsigned char usbConfigured=FALSE;

extern volatile unsigned int UsbInPending;
extern volatile unsigned int UsbOutPending;

unsigned char timeoutACounts=0;
unsigned char timeoutBCounts=0;
unsigned char timeoutCCounts=0;
unsigned char timeoutDCounts=0;
unsigned char fcCounts=0;
unsigned char timeoutUSBCounts=0;
unsigned char timeoutOccurred=0;

unsigned char systemState=0x00;
unsigned char lasersDetected=0x00; // Binary 1=detected,0=not detected, Bit [0] is Laser A, [1] is Laser B, [2] is Laser C, [3] is Laser D
unsigned char theChosenOne=0x00; //The laser found to be present and is queried for global things like baseplate temp, interlock, etc
unsigned char debugIteration=0;

#define EXPLORER_16 //The Stradus MultiLaser Board is functionally similar to the PIC EXPLORER 16 Dev Board

#define PLL96DIS_ON 0xF7FF //Somehow the compiler cannot find this in the device header files...strange.

#if defined(EXPLORER_16)
    #if defined(__PIC24FJ256GB110__) //Defined by MPLAB when using 24FJ256GB110 device was xxxxxx110 now xxxxxx106
        _CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF & ICS_PGx2)
        _CONFIG2( PLL96DIS_ON & IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_ON & POSCMOD_HS & FNOSC_PRIPLL & PLLDIV_DIV2 & IOL1WAY_ON)
    #elif defined(__PIC24FJ256GB106__) //Defined by MPLAB when using 24FJ256GB106 device was xxxxxx110 now xxxxxx106
		//For Watchdog Disabled:
        //_CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF & ICS_PGx2) 
		//For Watchdog Enabled:
		#if defined(WATCHDOG)
			_CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_ON & ICS_PGx2 & WDTPS_PS4096)
		#else
			_CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF & ICS_PGx2)
		#endif
		_CONFIG2( PLL96DIS_ON & IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_ON & POSCMOD_HS & FNOSC_PRIPLL & PLLDIV_DIV2 & IOL1WAY_ON)       	
    #else
        #error No hardware board defined, see "HardwareProfile.h" and __FILE__
    #endif
#else
    #error No hardware board defined, see "HardwareProfile.h"
#endif

/*/ VARIABLES ******************************************************/
#pragma udata

unsigned char OUTPacket[64];	//User application buffer for receiving and holding OUT packets sent from the host
unsigned char INPacket[64];		//User application buffer for sending IN packets to the host
#pragma udata
BOOL blinkStatusValid;
USB_HANDLE USBGenericOutHandle;
USB_HANDLE USBGenericInHandle;
#pragma udata

/** PRIVATE PROTOTYPES *********************************************/
static void InitializeSystem(void);
void USBDeviceTasks(void);
void YourHighPriorityISRCode(void);
void YourLowPriorityISRCode(void);
void UserInit(void);
void ProcessIO(void);
void BlinkUSBStatus(void);

/** DECLARATIONS ***************************************************/
#pragma code

int main(void)
{			
	initBoard();
			
    InitializeSystem();

	InitI2C();
	extUARTInit();
	initEEPROM();
	if(isEepromWorking==FALSE)
	{
		sprintf(serOutBuf,"\r\n***EEPROM ERROR***");
		sendString(EXT_UART,serOutBuf);
	}
	initShadowRegs(); //Non-Volatile Registers
	sysCfgInit(); //Registers loaded from EEPROM

	if(isCalImageGood==FALSE)
	{
		sprintf(serOutBuf,"\r\n***CAL ERROR***");
		sendString(EXT_UART,serOutBuf);
	}
	if(isSysCfgImageGood==FALSE)
	{
		sprintf(serOutBuf,"\r\n***SYS CFG ERROR***");
		sendString(EXT_UART,serOutBuf);
	}
        
    qInit(); //Remove Later
    initLaserUARTs(); //Initialize UARTs only for lasers with specified laser types      

	powerUpDelay();  //Don't let User ask command or GUI connect by USB until Stradus Lasers have booted fully
    
    clearInputBuffs();
	PORTDbits.RD11=1; //Enable RS-232 Transciever Level Shifters after a short delay to avoid crashing Stradus firmware
	 
    initTimer1(1); //1ms tics
    startTimer1();
	
#if defined(USB_INTERRUPT)
    USBDeviceAttach();
#endif

#if defined(WATCHDOG)
	ClrWdt(); //Clear the Watchdog Timer
#endif
     
    while(1)
    {
#if defined(WATCHDOG)
		ClrWdt(); //Clear the Watchdog Timer
#endif

		mySysCounts=getTimer1Ticks();
		if((mySysCounts - myPrevSysCounts)>10) //10ms ticks
		{
			myPrevSysCounts=mySysCounts;
			mySysCounts=0;
			systemProcess(); //System Controller Process, (Disabled until fully functional)
		}
					  	

	    processFrontSerialEvents(USB_PORT); //Front-End USB Port
	    	
		getBytesFromUART(); //Front-End RS-232 Check External FIFO
				
		processFrontSerialEvents(EXT_UART); //Front-End RS-232
		
	    processBackSerialEvents(UART_1); //Laser A	      
	    processBackSerialEvents(UART_2); //Laser B
	    processBackSerialEvents(UART_3); //Laser C
	    processBackSerialEvents(UART_4); //Laser D
	 		
		myCounts=getTimer1Ticks();	    
    	if((myCounts-myPrevCounts)>1000) //1000 * Timer1 Period specified in initTimer1(period)
    	{
			LATBbits.LATB13^=1; //Toggle Test Point "WP1" on VersaLase PIC Board
	   	 	myPrevCounts=myCounts;		   				   	
			myCounts=0;			
		
			//*******Laser A Timeout*******
			if(A_Response_Pending==1)
			{
				if(timeoutACounts==5)
				{
					timeoutACounts=0;
					A_Response_Pending=0;
					GLOBAL_Response_Pending=0;
					clearGlobalResponseBuffers();
					shadowRegs.A_qFC[0]='T';
					shadowRegs.A_qFC[1]='\0';
	   				timeoutOccurred=1;
					setLaserAsUndetected(UART_1);
					if(calShadowRegs.laserTypeA>0)
					{
						sprintf(serOutBuf,"\r\n***TIMEOUT A***");
						sendString(EXT_UART,serOutBuf);
					}
					
				}
				else
				{
					timeoutACounts++;
				}
			}
			else
			{
				timeoutACounts=0;	
			}	
			//*******Laser B Timeout********
			if(B_Response_Pending==1)
			{
				if(timeoutBCounts==5)
				{
					timeoutBCounts=0;
					B_Response_Pending=0;
					GLOBAL_Response_Pending=0;
					clearGlobalResponseBuffers();
					shadowRegs.B_qFC[0]='T';
					shadowRegs.B_qFC[1]='\0'; 
	   				timeoutOccurred=1;
					setLaserAsUndetected(UART_2);
					if(calShadowRegs.laserTypeB>0)
					{
						sprintf(serOutBuf,"\r\n***TIMEOUT B***");
						sendString(EXT_UART,serOutBuf);
					}
				}
				else
				{
					timeoutBCounts++;
				}
			}
			else
			{
				timeoutBCounts=0;	
			}	
			//*******Laser C Timeout********
			if(C_Response_Pending==1)
			{
				if(timeoutCCounts==5)
				{
					timeoutCCounts=0;
					C_Response_Pending=0;
					GLOBAL_Response_Pending=0;
					clearGlobalResponseBuffers();
					shadowRegs.C_qFC[0]='T';
					shadowRegs.C_qFC[1]='\0';
	   				timeoutOccurred=1;
					setLaserAsUndetected(UART_3);
					if(calShadowRegs.laserTypeC>0)
					{
						sprintf(serOutBuf,"\r\n***TIMEOUT C***");
						sendString(EXT_UART,serOutBuf);
					}
				}
				else
				{
					timeoutCCounts++;
				}
			}
			else
			{
				timeoutCCounts=0;	
			}
			//*******Laser D Timeout********
			if(D_Response_Pending==1)
			{
				if(timeoutDCounts==5)
				{
					timeoutDCounts=0;
					D_Response_Pending=0;
					GLOBAL_Response_Pending=0;
					clearGlobalResponseBuffers();
					shadowRegs.D_qFC[0]='T';
					shadowRegs.D_qFC[1]='\0';
	   				timeoutOccurred=1;
					setLaserAsUndetected(UART_4);
					if(calShadowRegs.laserTypeD>0)
					{
						sprintf(serOutBuf,"\r\n***TIMEOUT D***");
						sendString(EXT_UART,serOutBuf);
					}
				}
				else
				{
					timeoutDCounts++;
				}
			}
			else
			{
				timeoutDCounts=0;	
			}		
			if(timeoutOccurred==1)
			{
				timeoutOccurred=0;	
				composePrompt(); //Display Prompt Once Even For Multiple Timeouts Occuring On Same User Command
			}															   					 		 								 
		} 		
				
	    	
        #if defined(USB_POLLING)
		// Check bus status and service USB interrupts.
        USBDeviceTasks(); // Interrupt or polling method.  If using polling, must call
        				  // this function periodically.  This function will take care
        				  // of processing and responding to SETUP transactions 
        				  // (such as during the enumeration process when you first
        				  // plug in).  USB hosts require that USB devices should accept
        				  // and process SETUP packets in a timely fashion.  Therefore,
        				  // when using polling, this function should be called 
        				  // frequently (such as once about every 100 microseconds) at any
        				  // time that a SETUP packet might reasonably be expected to
        				  // be sent by the host to your device.  In most cases, the
        				  // USBDeviceTasks() function does not take very long to
        				  // execute (~50 instruction cycles) before it returns.
        #endif
    				  
     
    }//end while
}//end main




void systemProcess()
{
	switch(systemState)
	{
		case IDLE: //Power-up Idle state
			if(sysStateCounter>50) //0.5sec after loop is entered
			{
				systemState=BEGIN_COMM_CHECK;
				sysStateCounter=0;
				displayBanner();
			}
			else
			{
				sysStateCounter++;
			}
			break;
		case BEGIN_COMM_CHECK: //Poll lasers to see which are present/vacant
			configuring=TRUE;
			setLaserAsUndetected(UART_1);
			setLaserAsUndetected(UART_2);
			setLaserAsUndetected(UART_3);
			setLaserAsUndetected(UART_4);
			//Query all lasers
			detectLasers();
			systemState=WAIT_COMM_CHECK;
			sysStateCounter=0;			
			break;
		case WAIT_COMM_CHECK:
			//Wait for lasers to reply, if check went ok go to normal
			if(sysStateCounter>100) //100 * 10ms = 1sec
			{		
				sysStateCounter=0;
				systemState=CHECK_LASERS;
				sprintf(serOutBuf,"\r\nChecking Configuration...");
				sendString(EXT_UART,serOutBuf);
			}
			else
				sysStateCounter++;
			break;
		case CHECK_LASERS:
			//Compare lasersDetected with expected laser types from EEPROM
			if(calShadowRegs.laserTypeA>0 && !(lasersDetected & 0x01))
			{
				sprintf(serOutBuf,"\r\nLaser A Not Communicating");
				sendString(EXT_UART,serOutBuf);
			}
			if(calShadowRegs.laserTypeB>0 && !(lasersDetected & 0x02))
			{
				sprintf(serOutBuf,"\r\nLaser B Not Communicating");
				sendString(EXT_UART,serOutBuf);
			}
			if(calShadowRegs.laserTypeC>0 && !(lasersDetected & 0x04))
			{
				sprintf(serOutBuf,"\r\nLaser C Not Communicating");
				sendString(EXT_UART,serOutBuf);
			}
			if(calShadowRegs.laserTypeD>0 && !(lasersDetected & 0x08))
			{
				sprintf(serOutBuf,"\r\nLaser D Not Communicating");
				sendString(EXT_UART,serOutBuf);
			}
			systemState=READY_FOR_INPUT;			 
			break;
		case READY_FOR_INPUT:			
			systemState=NORMAL;
			configuring=FALSE;
			composePrompt();			
			break;
		case NORMAL: //Normally functioning state
			break;
	}
	
}

void powerUpDelay()
{	
	initTimer1(1);
	startTimer1();
	while(1)
	{	
		myCounts=getTimer1Ticks();	    
	    if((myCounts-myPrevCounts)>1000) //1000 * Timer1 Period specified in initTimer1(period)
	    {    
		   	myPrevCounts=myCounts;		   				   	
			myCounts=0;	
			if(fcCounts==6) //6 seconds
			{
				fcCounts=0;
				break;
			}
			else
			{
				fcCounts++;
			}
		}						
	}
	stopTimer1();
}	

void clearInputBuffs()
{
	while(U1RX_Counter>0)
	{       
   		k1 = U1bgetc();	 		
 	}  
 	memset(lineBuff1,0x00,U1_LINE_BUFF_SIZE);	
	lineBuffIndex1=0;

	while(U2RX_Counter>0)
	{       
   		k2 = U2bgetc();	 		
 	}  
 	memset(lineBuff2,0x00,U2_LINE_BUFF_SIZE);
	lineBuffIndex2=0;
 	while(U3RX_Counter>0)
	{       
   		k3 = U3bgetc();	
 	}  
	memset(lineBuff3,0x00,U3_LINE_BUFF_SIZE);
	lineBuffIndex3=0;
 	while(U1RX_Counter>0)
	{       
   		k4 = U4bgetc();	
 	}  	
	memset(lineBuff4,0x00,U4_LINE_BUFF_SIZE);	
	lineBuffIndex4=0;	
}	

void setLaserAsDetected(unsigned char laser)
{
	switch(laser)
	{	
		case UART_1: //Laser A
			lasersDetected |= 0x01;
			if(calShadowRegs.laserTypeA==LSR_STRADUS)
				if(theChosenOne==0x00)
					theChosenOne=laser;					
			break;
		case UART_2: //Laser B
			lasersDetected |= 0x02;
			if(calShadowRegs.laserTypeB==LSR_STRADUS)
				if(theChosenOne==0x00)
					theChosenOne=laser;	
			break;
		case UART_3: //Laser C
			lasersDetected |= 0x04;
			if(calShadowRegs.laserTypeC==LSR_STRADUS)
				if(theChosenOne==0x00)
					theChosenOne=laser;	
			break;
		case UART_4: //Laser D
			lasersDetected |= 0x08;
			if(calShadowRegs.laserTypeD==LSR_STRADUS)
				if(theChosenOne==0x00)
					theChosenOne=laser;	
			break;
	}		
}

void setLaserAsUndetected(unsigned char laser)
{
	switch(laser)
	{	
		case UART_1: //Laser A
			lasersDetected &= ~0x01;
			break;
		case UART_2: //Laser B
			lasersDetected &= ~0x02;
			break;
		case UART_3: //Laser C
			lasersDetected &= ~0x04;
			break;
		case UART_4: //Laser D
			lasersDetected &= ~0x08;
			break;
	}	
	if(theChosenOne==laser)
		theChosenOne=0x00;
}


unsigned char isSystemReady()
{
	//Are any messages from lasers pending or is an incoming (USB OUT) USB message pending
	if(A_Response_Pending || B_Response_Pending || C_Response_Pending || D_Response_Pending || GLOBAL_Response_Pending)
	{
		return FALSE;
	}
	else
	{
		return TRUE;	
	}		
}	

//Process Incoming Events on USB, UART1 lines
void processFrontSerialEvents(unsigned char myPort)
{
	static unsigned char mySerUExtData[MAX_SERIAL_BUF_SIZE];
	unsigned char myUExtPos;
	char parseUExt=0x00;
	static unsigned char SerUExtIndex;	
	
	if(myPort == EXT_UART)
	{				
		if(mySerialUExtPos>0)
		{    
			for(myUExtPos=0;myUExtPos<mySerialUExtPos;myUExtPos++)
			{
				mySerUExtData[SerUExtIndex++] = mySerialUExtIn[myUExtPos];
				if(mySerialUExtIn[myUExtPos]==0x0D)
				{
					if(isSystemReady())
					{
						parseUExt=0x01;
						break;
					}
				}
				else
					if(mySerialUExtIn[myUExtPos]!=0x0A)
	   				{	   					   			   				
		   	   			if(sysCfgShadowRegs.Echo==1)
		   	   			{
		   	   				sprintf(myStringExt,"%c",mySerialUExtIn[myUExtPos]); // and echo it back	
		   	   				sendString(EXT_UART,myStringExt); 
		   	   			}		   	   			   	   	   		
					}			
			}
			mySerialUExtPos=0;   	 
  		}
		if(parseUExt==0x01)
		{
			parseUExt=0x00;
			setLaserAsDetected(EXT_UART); //A newline was rx'd so it's alive

			parser(EXT_UART,(char*)mySerUExtData,EXT_UART_Q,SerUExtIndex);
			for(myUExtPos=0;myUExtPos<SerUExtIndex;myUExtPos++)
			{
				mySerUExtData[myUExtPos]=0x00;
			}
			SerUExtIndex=0;			
		}		
	} 
	else
	{
		if(myPort == USB_PORT)
		{	
			if(UsbOutPending==0x0001)
			{				
				if(isSystemReady())
				{
					UsbOutPending=0x0000;
					parser(USB_PORT,(char*)myUsbOutTmpBuf, USB_Q, (unsigned char)strlen((char*)myUsbOutTmpBuf));	
					memset(myUsbOutTmpBuf,0,sizeof(myUsbOutTmpBuf));
				}					
			}	
		}

	}				
}	

void clearInputBuff(unsigned char myPort)
{
	if(myPort==UART_1)
	{	
   		memset(lineBuff1,0x00,U1_LINE_BUFF_SIZE);	
   		lineBuffIndex1=0;
	}
	else
		if(myPort==UART_2)
		{	
   			memset(lineBuff2,0x00,U2_LINE_BUFF_SIZE);	
   			lineBuffIndex2=0;
		}
		else
				if(myPort==UART_3)
				{	
   					memset(lineBuff2,0x00,U3_LINE_BUFF_SIZE);	
   					lineBuffIndex3=0;
				}
				else
					if(myPort==UART_4)
					{	
   						memset(lineBuff2,0x00,U4_LINE_BUFF_SIZE);	
   						lineBuffIndex4=0;
					}
}



void processBackSerialEvents(unsigned char myPort)
{
	static unsigned char mySerU1Data[MAX_SERIAL_BUF_SIZE];
	unsigned char myU1Pos;
	char parseU1=0x00;
	static unsigned char SerU1Index;
	
	static unsigned char mySerU2Data[MAX_SERIAL_BUF_SIZE];
	unsigned char myU2Pos;
	char parseU2=0x00;
	static unsigned char SerU2Index;

	static unsigned char mySerU3Data[MAX_SERIAL_BUF_SIZE];
	unsigned char myU3Pos;
	char parseU3=0x00;
	static unsigned char SerU3Index;

	static unsigned char mySerU4Data[MAX_SERIAL_BUF_SIZE];
	unsigned char myU4Pos;
	char parseU4=0x00;
	static unsigned char SerU4Index;

	if(myPort==UART_1)
	{
		if(mySerialU1Pos>0)
		{
			//Disable Interrupt
			IEC0bits.U1RXIE = 0;	// Disable Receive Interrupts
			for(myU1Pos=0;myU1Pos<mySerialU1Pos;myU1Pos++)
			{
				mySerU1Data[SerU1Index++] = mySerialU1In[myU1Pos];
				if(mySerialU1In[myU1Pos]==0x0A)
				{
					parseU1=0x01;
					break;
				}
			}
			mySerialU1Pos=0;
			//Enable Interrupt
			IEC0bits.U1RXIE = 1;	// Enable Receive Interrupts
		}
		if(parseU1==0x01)
		{
			parseU1=0x00;
			setLaserAsDetected(UART_1); //A newline was rx'd so it's alive

			if(calShadowRegs.laserTypeA==LSR_STRADUS)
			{
   				backEndparser(EXT_UART,UART_1, (char*)mySerU1Data, SerU1Index);
			}
			else 
				if(calShadowRegs.laserTypeA==LSR_OXXIUS)
				{
					backEndparserOx(EXT_UART,UART_1, (char*)mySerU1Data, SerU1Index);
				}
			for(myU1Pos=0;myU1Pos<SerU1Index;myU1Pos++)
			{
				mySerU1Data[myU1Pos]=0x00;
			}
			SerU1Index=0;			
		}			
	}
	else	
		if(myPort==UART_2)
		{		
			if(mySerialU2Pos>0)
			{
				//Disable Interrupt
				IEC1bits.U2RXIE = 0;	// Disable Receive Interrupts
				for(myU2Pos=0;myU2Pos<mySerialU2Pos;myU2Pos++)
				{
					mySerU2Data[SerU2Index++] = mySerialU2In[myU2Pos];
					if(mySerialU2In[myU2Pos]==0x0A)
					{
						parseU2=0x01;
						break;
					}
				}
				mySerialU2Pos=0;
				//Enable Interrupt
				IEC1bits.U2RXIE = 1;	// Enable Receive Interrupts
			}
			if(parseU2==0x01)
			{
				parseU2=0x00;
				setLaserAsDetected(UART_2); //A newline was rx'd so it's alive
			
				if(calShadowRegs.laserTypeB==LSR_STRADUS)
				{
					backEndparser(EXT_UART,UART_2, (char*)mySerU2Data, SerU2Index);
				}
				else 
					if(calShadowRegs.laserTypeB==LSR_OXXIUS)
					{
						backEndparserOx(EXT_UART,UART_2, (char*)mySerU2Data, SerU2Index);
					}
				for(myU2Pos=0;myU2Pos<SerU2Index;myU2Pos++)
				{
					mySerU2Data[myU2Pos]=0x00;
				}
				SerU2Index=0;			
			}  				
		}	
		else
			if(myPort==UART_3)
			{
				if(mySerialU3Pos>0)
				{
					//Disable Interrupt
					IEC5bits.U3RXIE = 0;	// Disable Receive Interrupts
					for(myU3Pos=0;myU3Pos<mySerialU3Pos;myU3Pos++)
					{
						//sprintf(myStringExt,"%c",mySerialU3In[myU3Pos]); // debug	
		   	   			//sendString(EXT_UART,myStringExt); 
						mySerU3Data[SerU3Index++] = mySerialU3In[myU3Pos];
						
						if(mySerialU3In[myU3Pos]==0x0A)
						{
							parseU3=0x01;
							break;
						}		
					}
					mySerialU3Pos=0;
					//Enable Interrupt
					IEC5bits.U3RXIE = 1;	// Enable Receive Interrupts
				}
				if(parseU3==0x01)
				{
					parseU3=0x00;
					setLaserAsDetected(UART_3); //A newline was rx'd so it's alive
				
					if(calShadowRegs.laserTypeC==LSR_STRADUS)
					{
						//sprintf(myStringExt,"%s",mySerU3Data); // debug	
		   	   			//sendString(EXT_UART,myStringExt); 
						backEndparser(EXT_UART,UART_3, (char*)mySerU3Data, SerU3Index);						
					}
					else 
						if(calShadowRegs.laserTypeC==LSR_OXXIUS)
						{
							backEndparserOx(EXT_UART,UART_3, (char*)mySerU3Data, SerU3Index);
						}
					for(myU3Pos=0;myU3Pos<SerU3Index;myU3Pos++)
					{
						mySerU3Data[myU3Pos]=0x00;
					}
					SerU3Index=0;			
				}			
			}	
			else 
				if(myPort==UART_4)
				{
					if(mySerialU4Pos>0)
					{
						//Disable Interrupt
						IEC5bits.U4RXIE = 0;	// Disable Receive Interrupts
						for(myU4Pos=0;myU4Pos<mySerialU4Pos;myU4Pos++)
						{
							mySerU4Data[SerU4Index++] = mySerialU4In[myU4Pos];
							if(mySerialU4In[myU4Pos]==0x0A)
							{
								parseU4=0x01;
								break;
							}
						}
						mySerialU4Pos=0;
						//Enable Interrupt
						IEC5bits.U4RXIE = 1;	// Enable Receive Interrupts
					}
					if(parseU4==0x01)
					{
						parseU4=0x00;
						setLaserAsDetected(UART_4); //A newline was rx'd so it's alive
					
						if(calShadowRegs.laserTypeD==LSR_STRADUS)
						{
							backEndparser(EXT_UART,UART_4, (char*)mySerU4Data, SerU4Index);
						}
						else 
							if(calShadowRegs.laserTypeD==LSR_OXXIUS)
							{
								backEndparserOx(EXT_UART,UART_4, (char*)mySerU4Data, SerU4Index);
							}
						for(myU4Pos=0;myU4Pos<SerU4Index;myU4Pos++)
						{
							mySerU4Data[myU4Pos]=0x00;
						}
						SerU4Index=0;			
					}				
				}		
				
}	

/********************************************************************
 * Function:        static void InitializeSystem(void)
 *
 *
 * Overview:        InitializeSystem is a centralized initialization
 *                  routine. All required USB initialization routines
 *                  are called from here.
 *
 *******************************************************************/
static void InitializeSystem(void)
{
    #if (defined(__18CXX) & !defined(PIC18F87J50_PIM))
        ADCON1 |= 0x0F;                 // Default all pins to digital
    #elif defined(__C30__)
        AD1PCFGL = 0xFFFF; //Must do this DMS!!!
    #elif defined(__C32__)
        AD1PCFG = 0xFFFF;
    #endif

    #if defined(PIC18F87J50_PIM) || defined(PIC18F46J50_PIM) || defined(PIC18F_STARTER_KIT_1)
	//On the PIC18F87J50 Family of USB microcontrollers, the PLL will not power up and be enabled
	//by default, even if a PLL enabled oscillator configuration is selected (such as HS+PLL).
	//This allows the device to power up at a lower initial operating frequency, which can be
	//advantageous when powered from a source which is not gauranteed to be adequate for 48MHz
	//operation.  On these devices, user firmware needs to manually set the OSCTUNE<PLLEN> bit to
	//power up the PLL.
    {
        unsigned int pll_startup_counter = 600;
        OSCTUNEbits.PLLEN = 1;  //Enable the PLL and wait 2+ms until the PLL locks before enabling USB module
        while(pll_startup_counter--);
    }
    //Device switches over automatically to PLL output after PLL is locked and ready.
    #endif
    
   #if defined(PIC24FJ64GB004_PIM)
	//On the PIC24FJ64GB004 Family of USB microcontrollers, the PLL will not power up and be enabled
	//by default, even if a PLL enabled oscillator configuration is selected (such as HS+PLL).
	//This allows the device to power up at a lower initial operating frequency, which can be
	//advantageous when powered from a source which is not gauranteed to be adequate for 32MHz
	//operation.  On these devices, user firmware needs to manually set the CLKDIV<PLLEN> bit to
	//power up the PLL.
    {
        unsigned int pll_startup_counter = 600;
        CLKDIVbits.PLLEN = 1;
        while(pll_startup_counter--);
    }

    //Device switches over automatically to PLL output after PLL is locked and ready.
    #endif


//	The USB specifications require that USB peripheral devices must never source
//	current onto the Vbus pin.  Additionally, USB peripherals should not source
//	current on D+ or D- when the host/hub is not actively powering the Vbus line.
//	When designing a self powered (as opposed to bus powered) USB peripheral
//	device, the firmware should make sure not to turn on the USB module and D+
//	or D- pull up resistor unless Vbus is actively powered.  Therefore, the
//	firmware needs some means to detect when Vbus is being powered by the host.
//	A 5V tolerant I/O pin can be connected to Vbus (through a resistor), and
// 	can be used to detect when Vbus is high (host actively powering), or low
//	(host is shut down or otherwise not supplying power).  The USB firmware
// 	can then periodically poll this I/O pin to know when it is okay to turn on
//	the USB module/D+/D- pull up resistor.  When designing a purely bus powered
//	peripheral device, it is not possible to source current on D+ or D- when the
//	host is not actively providing power on Vbus. Therefore, implementing this
//	bus sense feature is optional.  This firmware can be made to use this bus
//	sense feature by making sure "USE_USB_BUS_SENSE_IO" has been defined in the
//	HardwareProfile.h file.    
    #if defined(USE_USB_BUS_SENSE_IO)
    tris_usb_bus_sense = INPUT_PIN; // See HardwareProfile.h
    #endif
    
//	If the host PC sends a GetStatus (device) request, the firmware must respond
//	and let the host know if the USB peripheral device is currently bus powered
//	or self powered.  See chapter 9 in the official USB specifications for details
//	regarding this request.  If the peripheral device is capable of being both
//	self and bus powered, it should not return a hard coded value for this request.
//	Instead, firmware should check if it is currently self or bus powered, and
//	respond accordingly.  If the hardware has been configured like demonstrated
//	on the PICDEM FS USB Demo Board, an I/O pin can be polled to determine the
//	currently selected power source.  On the PICDEM FS USB Demo Board, "RA2" 
//	is used for	this purpose.  If using this feature, make sure "USE_SELF_POWER_SENSE_IO"
//	has been defined in HardwareProfile.h, and that an appropriate I/O pin has been mapped
//	to it in HardwareProfile.h.
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;	// See HardwareProfile.h
    #endif
    
	USBGenericOutHandle = 0;	
	USBGenericInHandle = 0;		

    UserInit();			//Application related initialization.  See user.c

    USBDeviceInit();	//usb_device.c.  Initializes USB module SFRs and firmware
    					//variables to known states.
}//end InitializeSystem



void UserInit(void)
{
    //mInitAllLEDs();
    //mInitAllSwitches();

	blinkStatusValid = TRUE;	//Blink the normal USB state on the LEDs.

}//end UserInit


// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
// ******************************************************************************************************
// The USB firmware stack will call the callback functions USBCBxxx() in response to certain USB related
// events.  For example, if the host PC is powering down, it will stop sending out Start of Frame (SOF)
// packets to your device.  In response to this, all USB devices are supposed to decrease their power
// consumption from the USB Vbus to <2.5mA each.  The USB module detects this condition (which according
// to the USB specifications is 3+ms of no bus activity/SOF packets) and then calls the USBCBSuspend()
// function.  You should modify these callback functions to take appropriate actions for each of these
// conditions.  For example, in the USBCBSuspend(), you may wish to add code that will decrease power
// consumption from Vbus to <2.5mA (such as by clock switching, turning off LEDs, putting the
// microcontroller to sleep, etc.).  Then, in the USBCBWakeFromSuspend() function, you may then wish to
// add code that undoes the power saving things done in the USBCBSuspend() function.

// The USBCBSendResume() function is special, in that the USB stack will not automatically call this
// function.  This function is meant to be called from the application firmware instead.  See the
// additional comments near the function.

/******************************************************************************
 * Function:        void USBCBSuspend(void)
 *
 * Overview:        Call back that is invoked when a USB suspend is detected
 *
 * Note:            None
 *****************************************************************************/
void USBCBSuspend(void)
{
	//Example power saving code.  Insert appropriate code here for the desired
	//application behavior.  If the microcontroller will be put to sleep, a
	//process similar to that shown below may be used:
	
	//ConfigureIOPinsForLowPower();
	//SaveStateOfAllInterruptEnableBits();
	//DisableAllInterruptEnableBits();
	//EnableOnlyTheInterruptsWhichWillBeUsedToWakeTheMicro();	//should enable at least USBActivityIF as a wake source
	//Sleep();
	//RestoreStateOfAllPreviouslySavedInterruptEnableBits();	//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.
	//RestoreIOPinsToNormal();									//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.

	//IMPORTANT NOTE: Do not clear the USBActivityIF (ACTVIF) bit here.  This bit is 
	//cleared inside the usb_device.c file.  Clearing USBActivityIF here will cause 
	//things to not work as intended.	
	

    #if defined(__C30__)
    #if 0
        U1EIR = 0xFFFF;
        U1IR = 0xFFFF;
        U1OTGIR = 0xFFFF;
        IFS5bits.USB1IF = 0;
        IEC5bits.USB1IE = 1;
        U1OTGIEbits.ACTVIE = 1;
        U1OTGIRbits.ACTVIF = 1;
        Sleep();
    #endif
    #endif
}


/******************************************************************************
 * Function:        void _USB1Interrupt(void)
 *
 *
 * Overview:        This function is called when the USB interrupt bit is set
 *					In this example the interrupt is only used when the device
 *					goes to sleep when it receives a USB suspend command
 *
 * Note:            None
 *****************************************************************************/
#if 0
void __attribute__ ((interrupt)) _USB1Interrupt(void)
{
    #if !defined(self_powered)
        if(U1OTGIRbits.ACTVIF)
        {
            IEC5bits.USB1IE = 0;
            U1OTGIEbits.ACTVIE = 0;
            IFS5bits.USB1IF = 0;
        
            //USBClearInterruptFlag(USBActivityIFReg,USBActivityIFBitNum);
            USBClearInterruptFlag(USBIdleIFReg,USBIdleIFBitNum);
            //USBSuspendControl = 0;
        }
    #endif
}
#endif

/******************************************************************************
 * Function:        void USBCBWakeFromSuspend(void)
 *
 * Overview:        The host may put USB peripheral devices in low power
 *					suspend mode (by "sending" 3+ms of idle).  Once in suspend
 *					mode, the host may wake the device back up by sending non-
 *					idle state signalling.
 *					
 *					This call back is invoked when a wakeup from USB suspend 
 *					is detected.
 *
 * Note:            None
 *****************************************************************************/
void USBCBWakeFromSuspend(void)
{
	// If clock switching or other power savings measures were taken when
	// executing the USBCBSuspend() function, now would be a good time to
	// switch back to normal full power run mode conditions.  The host allows
	// a few milliseconds of wakeup time, after which the device must be 
	// fully back to normal, and capable of receiving and processing USB
	// packets.  In order to do this, the USB module must receive proper
	// clocking (IE: 48MHz clock must be available to SIE for full speed USB
	// operation).
}

/********************************************************************
 * Function:        void USBCB_SOF_Handler(void)
 *
 *
 * Overview:        The USB host sends out a SOF packet to full-speed
 *                  devices every 1 ms. This interrupt may be useful
 *                  for isochronous pipes. End designers should
 *                  implement callback routine as necessary.
 *
 * Note:            None
 *******************************************************************/
void USBCB_SOF_Handler(void)
{
    // No need to clear UIRbits.SOFIF to 0 here.
    // Callback caller is already doing that.
}

/*******************************************************************
 * Function:        void USBCBErrorHandler(void)
 *
 *
 * Overview:        The purpose of this callback is mainly for
 *                  debugging during development. Check UEIR to see
 *                  which error causes the interrupt.
 *
 * Note:            None
 *******************************************************************/
void USBCBErrorHandler(void)
{
    // No need to clear UEIR to 0 here.
    // Callback caller is already doing that.

	// Typically, user firmware does not need to do anything special
	// if a USB error occurs.  For example, if the host sends an OUT
	// packet to your device, but the packet gets corrupted (ex:
	// because of a bad connection, or the user unplugs the
	// USB cable during the transmission) this will typically set
	// one or more USB error interrupt flags.  Nothing specific
	// needs to be done however, since the SIE will automatically
	// send a "NAK" packet to the host.  In response to this, the
	// host will normally retry to send the packet again, and no
	// data loss occurs.  The system will typically recover
	// automatically, without the need for application firmware
	// intervention.
	
	// Nevertheless, this callback function is provided, such as
	// for debugging purposes.
}


/*******************************************************************
 * Function:        void USBCBCheckOtherReq(void)
 *
 *
 * Overview:        When SETUP packets arrive from the host, some
 * 					firmware must process the request and respond
 *					appropriately to fulfill the request.  Some of
 *					the SETUP packets will be for standard
 *					USB "chapter 9" (as in, fulfilling chapter 9 of
 *					the official USB specifications) requests, while
 *					others may be specific to the USB device class
 *					that is being implemented.  For example, a HID
 *					class device needs to be able to respond to
 *					"GET REPORT" type of requests.  This
 *					is not a standard USB chapter 9 request, and 
 *					therefore not handled by usb_device.c.  Instead
 *					this request should be handled by class specific 
 *					firmware, such as that contained in usb_function_hid.c.
 *
 * Note:            None
 *****************************************************************************/
void USBCBCheckOtherReq(void)
{
	//Handle non-USB specification requests such as Vendor Requests
	if(SetupPkt.RequestType==VENDOR) //VENDOR==0x02
	{	  
			inPipes[0].info.bits.busy = 1; //dms
			if (SetupPkt.DataDir==IN_TO_HOST) 
			{
	  	    	if (!VendorRequestInSetupHandler ()) //If not handled then stall endpoint
	  	    	{
      	      		//USBStallHandler();
      	      								// Returns FALSE if not handled
      	  		}    								
	      	}
		  	else //OUT_FROM_HOST
		  	{
				if(!VendorRequestOutSetupHandler()) //If not handled then stall endpoint
				{
					//USBStallHandler();
				}	
		  	}			  		  		   
	} 	  	
}//end


/*******************************************************************
 * Function:        void USBCBStdSetDscHandler(void)
 *
 *
 * Overview:        The USBCBStdSetDscHandler() callback function is
 *					called when a SETUP, bRequest: SET_DESCRIPTOR request
 *					arrives.  Typically SET_DESCRIPTOR requests are
 *					not used in most applications, and it is
 *					optional to support this type of request.
 *
 * Note:            None
 *****************************************************************************/
void USBCBStdSetDscHandler(void)
{
    // Must claim session ownership if supporting this request
}//end


/******************************************************************************
 * Function:        void USBCBInitEP(void)
 *
 *
 * Overview:        This function is called when the device becomes
 *                  initialized, which occurs after the host sends a
 * 					SET_CONFIGURATION (wValue not = 0) request.  This 
 *					callback function should initialize the endpoints 
 *					for the device's usage according to the current 
 *					configuration.
 *
 * Note:            None
 *****************************************************************************/
void USBCBInitEP(void)
{
    USBEnableEndpoint(USBGEN_EP_NUM,USB_OUT_ENABLED|USB_IN_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    USBGenericOutHandle = USBGenRead(USBGEN_EP_NUM,(BYTE*)&OUTPacket,USBGEN_EP_SIZE);
}

/********************************************************************
 * Function:        void USBCBSendResume(void)
 *
 *
 * Overview:        The USB specifications allow some types of USB
 * 					peripheral devices to wake up a host PC (such
 *					as if it is in a low power suspend to RAM state).
 *					This can be a very useful feature in some
 *					USB applications, such as an Infrared remote
 *					control	receiver.  If a user presses the "power"
 *					button on a remote control, it is nice that the
 *					IR receiver can detect this signalling, and then
 *					send a USB "command" to the PC to wake up.
 *					
 *					The USBCBSendResume() "callback" function is used
 *					to send this special USB signalling which wakes 
 *					up the PC.  This function may be called by
 *					application firmware to wake up the PC.  This
 *					function should only be called when:
 *					
 *					1.  The USB driver used on the host PC supports
 *						the remote wakeup capability.
 *					2.  The USB configuration descriptor indicates
 *						the device is remote wakeup capable in the
 *						bmAttributes field.
 *					3.  The USB host PC is currently sleeping,
 *						and has previously sent your device a SET 
 *						FEATURE setup packet which "armed" the
 *						remote wakeup capability.   
 *
 *					This callback should send a RESUME signal that
 *                  has the period of 1-15ms.
 *
 * Note:            Interrupt vs. Polling
 *                  -Primary clock
 *                  -Secondary clock ***** MAKE NOTES ABOUT THIS *******
 *                   > Can switch to primary first by calling USBCBWakeFromSuspend()
 
 *                  The modifiable section in this routine should be changed
 *                  to meet the application needs. Current implementation
 *                  temporary blocks other functions from executing for a
 *                  period of 1-13 ms depending on the core frequency.
 *
 *                  According to USB 2.0 specification section 7.1.7.7,
 *                  "The remote wakeup device must hold the resume signaling
 *                  for at lest 1 ms but for no more than 15 ms."
 *                  The idea here is to use a delay counter loop, using a
 *                  common value that would work over a wide range of core
 *                  frequencies.
 *                  That value selected is 1800. See table below:
 *                  ==========================================================
 *                  Core Freq(MHz)      MIP         RESUME Signal Period (ms)
 *                  ==========================================================
 *                      48              12          1.05
 *                       4              1           12.6
 *                  ==========================================================
 *                  * These timing could be incorrect when using code
 *                    optimization or extended instruction mode,
 *                    or when having other interrupts enabled.
 *                    Make sure to verify using the MPLAB SIM's Stopwatch
 *                    and verify the actual signal on an oscilloscope.
 *******************************************************************/
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    USBResumeControl = 1;                // Start RESUME signaling
    
    delay_count = 1800U;                // Set RESUME line for 1-13 ms
    do
    {
        delay_count--;
    }while(delay_count);
    USBResumeControl = 0;
}


/*******************************************************************
 * Function:        BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *                        USB_EVENT event, void *pdata, WORD size)
 *
 * Input:           USB_EVENT event - the type of event
 *                  void *pdata - pointer to the event data
 *                  WORD size - size of the event data
 *
 *
 * Overview:        This function is called from the USB stack to
 *                  notify a user application that a USB event
 *                  occured.  This callback is in interrupt context
 *                  when the USB_INTERRUPT option is selected.
 *
 * Note:            None
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER:
            Nop();
            break;
        default:
            break;
    }      
    return TRUE; 
}


/** EOF main.c ***************************************************************/
