/*
parser.c
Function: Front-End Parser to parse user commands received over serial
*/
#include "circular_queue.h"
#include "uart.h"
#include "parser.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "type.h"
#include "system.h"
#include "config.h"
#include "timers.h"
#include <math.h>

char myResponseBuf[MAX_CMD_SIZE];
char myBuf[MAX_CMD_RSP_SIZE];
static char myTmpBuf[MAX_CMD_SIZE];
char myCmdToken[MAX_CMD_SIZE];
char myPrefixToken[MAX_CMD_SIZE];
char myEndingToken[32];
unsigned char myLsrType=0;
char* arg1;
char* arg2;
char* expectedCmdReply;
static unsigned char myOutputCommPort;
unsigned char activeOutPort=NULL;
char isManufModeActive = FALSE;
unsigned char cmdToMatch[5];


unsigned char portToDevice=NULL;
unsigned char portToHost=NULL;

void strupr(char *myUprCaseBuf,char *myInputBuf);

extern char outgoing[256]; //Top Level Output Buffer
extern VOLATILE_REGISTERS shadowRegs;

extern unsigned char A_Response_Pending;
extern unsigned char B_Response_Pending;
extern unsigned char C_Response_Pending;
extern unsigned char D_Response_Pending;
extern unsigned char GLOBAL_Response_Pending;
extern unsigned char configuring;

extern unsigned char lasersDetected;

extern volatile unsigned char UsbInPending;
extern volatile unsigned char UsbOutPending;

const PROTOCOL_CMDS SystemCmds[] = {
// {Cmd String,Max Args,Prefix Req'd?,Function Ptr}
	{"C",				1,1,    cHandler}, //Individual
	{"DELAY", 	        1,1,	delayHandler}, //Individual
	{"ECHO", 			1,0,	echoHandler}, //Internal
	{"EPC", 			1,1,	epcHandler}, //Individual
	{"LE", 			    1,1,	leHandler}, //Individual
	{"LP", 			    1,1,	lpHandler}, //Individual
	{"PUL",             1,1,     pulHandler}, //Individual
	{"PP",              1,1,     ppHandler}, //Individual
	{"PROMPT",          1,0,    promptHandler}, //Individual
	{"TEC", 			1,1,	tecHandler}, //Individual
	{"VOBTS", 		    1,1,	vobtsHandler}, //Individual
			
	{"?BPT", 			0,0,	q_bptHandler}, //Ask the chosen Stradus
	{"?C",              0,1,     q_cHandler}, //Individual
	{"?CC",				0,1,    q_ccHandler}, //Individual
	{"?CT",             0,1,    q_ctHandler}, //Controller Temperature
	{"?DELAY", 			0,1,	q_delayHandler}, //Individual
	{"?EPC", 			0,1,	q_epcHandler}, //Individual
	{"?FC", 			0,1,	q_fcHandler}, //Individual
	{"?FD", 			0,1,	q_fdHandler}, //Individual
	{"?FP", 			0,1,	q_fpHandler}, //Individual
	{"?FV", 			0,1,	q_fvHandler}, //Individual
	{"?SPV", 			0,0,	q_spvHandler}, //Internal
	{"?SFV", 			0,0,	q_sfvHandler}, //Internal
	{"?H",              0,0,    q_hHandler}, //Internal
	{"?IL", 			0,0,	q_ilHandler}, //Ask the chosen Stradus
	{"?LC", 			0,1,	q_lcHandler}, //Individual
	{"?LE", 			0,1,	q_leHandler}, //Individual
	{"?LH", 			0,1,	q_lhHandler}, //Individual
	{"?LI", 			0,1,	q_liHandler}, //Individual
	{"?LP", 			0,1,	q_lpHandler}, //Individual
	{"?LPS", 			0,1,	q_lpsHandler}, //Individual
	{"?LW", 			0,1,	q_lwHandler}, //Individual
	{"?MAXP", 			0,1,	q_maxpHandler}, //Individual
	{"?OBT",			0,1,	q_obtHandler}, //Individual
	{"?OBTS", 			0,1,	q_obtsHandler}, //Individual
	{"?RP", 			0,1,	q_rpHandler}, //Individual
	{"?TEC", 			0,1,	q_tecHandler}, //Individual
	{"?PUL", 			0,1,	q_pulHandler}, //Individual
	{"?PP", 			0,1,	q_ppHandler}, //Individual
	{"?OP1",            0,1,    q_op1Handler}, //Individual
	{"?OP2",            0,1,    q_op2Handler}, //Individual
	{"?OP3",            0,1,    q_op3Handler}, //Individual
	{"?OP4",            0,1,    q_op4Handler}, //Individual
	{"?GUI1",           0,0,    q_gui1Handler}, //Internal

//Calibration Commands
	{"VORT",           0,0,    vortHandler}, //Internal
	{"BYPASS",         0,0,    bypassHandler}, //Internal
	{"BURN",           0,0,    burnHandler}, //Internal
	{"CONFIG",         0,0,    dsysCfgHandler}, //Internal
	{"TYPE",           1,1,    calTypeHandler}, //Internal
	{"LPN",            1,1,    calPartNumHandler}, //Individual
	{"LSN",            1,1,    calSerNumHandler}, //Individual
	{"LBS",            1,1,    calBeamShapeHandler}, //Individual
	{"NWL",            1,1,    calNomWaveLenHandler}, //Individual
	{"NLP", 		   1,1,	   calNomLsrPwrHandler}, //Individual
	{"MPL", 		   1,1,	   calMaxPwrLsrHandler}, //Individual
	{"VORT",           0,1,    calStradusMfrModeHandler}, //Individual
	{"?TYPE",          0,1,    q_calTypeHandler}, //Individual		
	{"",0,0,0} // DO NOT MOVE/REMOVE!!	END OF COMMANDS!!
};

const char helpTable[][MAX_CMD_RSP_SIZE] = {
	"HELP FILE:",
	" ",
	"VERSA-LASE SYSTEM COMMANDS/QUERIES:",
	"ECHO=[0/1].....Set Echo: Off/On",
	"PROMPT=[0/1].Set Prompt: Off/On",
	"?BPT........Get Base Plate Temp",
	"?H................Get Help File",
	"?IL........Get Interlock Status",
	"?SPV.......Get Protocol Version",
	"?SFV.......Get Firmware Version",
	" ",
	"LASER-SPECIFIC COMMANDS/QUERIES:",
	"(Use prefix ie. A.le=1 or B.?le)",
	"C=[0/1].........Set Control Mode",
	"DELAY=[0/1]....Toggle CDRH Delay",
	"EPC=[0/1].........Set EPC Off/On",
	"LE=[0/1].....Set Emission Off/On",
	"LP=###.#...Set Laser Power in mW",
	"PUL=[0/1]..Set Pulse Mode Off/On",
	"TEC=[0/1]..Set TE Cooler: Off/On",
	"?C..............Get Control Mode",
	"?CT...Get Controller Temperature",
	"?DELAY............Get CDRH Delay",
	"?EPC..............Get EPC Status",
	"?FC...............Get Fault Code",
	"?FD........Get Fault Description",
	"?FP...Get Laser Protocol Version",
	"?FV...Get Laser Firmware Version",
	"?LC............Get Laser Current",
	"?LE...........Get Laser Emission",
	"?LH....Get Laser Operating Hours",
	"?LP........Get Laser Power in mW",
	"?LPS.....Get Laser Power Setting",
	"?LW.........Get Laser Wavelength",
	"?MAXP....Get Maximum Laser Power",
	"?OBT..............Get Diode Temp",
	"?OBTS.....Get Diode Temp Setting",
	"?PUL.......Get Pulse Mode Status",
	"?RP........Get Rated Laser Power",
	"?TEC..............Get TEC Status",
	""		// END OF TABLE MARKER!! DO NOT REMOVE!!
};

//Prototypes
void setLaserStatus(unsigned char);

void initShadowRegs()
{
	shadowRegs.A_qFC[0]='-';
	shadowRegs.A_qFC[1]='\0';
	shadowRegs.B_qFC[0]='-';
	shadowRegs.B_qFC[1]='\0';
	shadowRegs.C_qFC[0]='-';
	shadowRegs.C_qFC[1]='\0';
	shadowRegs.D_qFC[0]='-';
	shadowRegs.D_qFC[1]='\0';		
}


void displayBanner()
{
	portToHost=EXT_UART;
	sprintf(outgoing,"\r\n\r\n********************");
	sendString(EXT_UART,outgoing);
    sprintf(outgoing,"\r\nStradus VersaLase\r\n(C) 2014 Vortran\r\nSW: V%s, HW: %s ",SW_VERSION,HW_VERSION);
	sendString(EXT_UART,outgoing);
	sprintf(outgoing,"\r\n********************\r\n");
	sendString(EXT_UART,outgoing);
}

//***********************************************************/
void strupr(char *myUprCaseBuf, char *myInputBuf)
{
	char myChar;
	volatile char *myTmp;

	myTmp = myUprCaseBuf;

	while(*myInputBuf)										// are there charcaters in the input buffer we need to process?
	{
		myChar = *myInputBuf;

		if(myChar >= 'a' && myChar <= 'z')	// is the character lower case?
		{
			myChar = *myInputBuf - 0x20;			 			// yes, convert to upper case!
			*myTmp = myChar;
		}
		else if(myChar >= 'A' && myChar <= 'Z')	// is the character upper case?
			*myTmp = myChar;
		else
			*myTmp = myChar;

		myTmp++;
		myInputBuf++;
	}
}

	
//************************************************************

// This parser accepts a stream of characters and determines if a valid command is present
// If valid then the command function/feature is performed and a specific output response is
// generated/composed for the specified stream. The "stream" argument is either Serial or USB.
void parser(char myPort, char *myBuf, Q_STREAM stream, unsigned char myMsgSize)
{
	unsigned char indx = 0;
	Q_STREAM myQ;
	char *myTmpPtr;
	char *myFinalPtr;
	unsigned char myLength=0;
	unsigned char myCmdLength=0;
	unsigned char myTmpLength=0;
	unsigned char myCnt = 0;
	unsigned char cmdMatched = 0;
	unsigned char charsRemoved = 0;
	unsigned char charsRemovedLeadingBS = 0;
	char	didWeFindCrKey = FALSE;
	char 	didWeFindEndOfCmdToken = FALSE;
	char    didWeFindEndOfPrefixToken = FALSE;

	myOutputCommPort = myPort; // save the current port	which generated the command
	portToHost=myPort;
	
	switch(myPort)
	{
		case USB_PORT:
			myQ = USB_Q;
			break;
		case UART_1:
			myQ = UART1_Q;
			break;
		case UART_2:
			myQ = UART2_Q;
			break;
		case UART_3:
			myQ = UART3_Q;
			break;
		case UART_4:
			myQ = UART4_Q;
			break;
		case EXT_UART:
			myQ = EXT_UART_Q;
			break;
		default:
			return;
	}

    if(*myBuf == CR_KEY || *myBuf == NL_KEY)	// Toss the enter key, or Toss the New Line key, and re-display the prompt
	{
		if(*myBuf == CR_KEY)
			composePrompt();
		return;
	}
	myCnt = 0;
	myTmpPtr = myBuf;

	while(*myTmpPtr == BS_KEY && myCnt++ < myMsgSize)	// remove all leading backspaces
	{
		myTmpPtr++;	// toss the Backspace
		charsRemovedLeadingBS++;
	}

	myFinalPtr = myTmpBuf; //Put ptr at beginning of myTmpBuff to operate on it

	// strip off backspace embedded in the input string
	for(myCnt = 0; myCnt < myMsgSize - charsRemovedLeadingBS; myCnt++)
	{
		if(*myTmpPtr == BS_KEY)
		{
			myFinalPtr--;	 // remove previous character
			*myFinalPtr = 0;
			myTmpPtr++;
			charsRemoved += 2;
		}
		else	// regular character
		{
			*myFinalPtr++ = *myTmpPtr++;	// save the character
		}
	}

	*myFinalPtr = 0; //Terminate string with NULL

	strcpy(myBuf, myTmpBuf);

	for(myCnt = 0; myCnt < MAX_CMD_SIZE; )	 // clear arrays
	{
		myTmpBuf[myCnt] = 0;
		myPrefixToken[myCnt]=0;
 		myCmdToken[myCnt++] = 0;
	}

	strupr(myPrefixToken,myBuf);         //convert command to upper case, store a copy

	myTmpPtr = myPrefixToken;
	myCnt=0;

	while(*myTmpPtr != CR_KEY && *myTmpPtr != 0)
	{
		if(*myTmpPtr == '.')
		{
			didWeFindEndOfPrefixToken = TRUE;
			*myTmpPtr=NULL; //End the Prefix Token, just contains one character
			break;
		}
		else
		{
			myTmpPtr++;
		}
	}
	
	if(didWeFindEndOfPrefixToken==TRUE)
	{
		strupr(myCmdToken,myBuf+2);		// convert command to upper case, store a copy, skip prefix
		strupr(myTmpBuf,myBuf+2);		    //convert command to upper case, store a copy
		if(*myPrefixToken=='a'||*myPrefixToken=='A')
		{
			activeOutPort=UART_1;
		}	
		else 
			if(*myPrefixToken=='b'||*myPrefixToken=='B')
			{
				activeOutPort=UART_2;
			}	
			else 
				if(*myPrefixToken=='c'||*myPrefixToken=='C')	
				{
					activeOutPort=UART_3;
				}
				else
					if(*myPrefixToken=='d'||*myPrefixToken=='D')	
					{
						activeOutPort=UART_4;
					}	
					else
					{
						if(*myPrefixToken==NULL)
						{
							activeOutPort=NULL;
						}	
					}

		switch(activeOutPort)
		{
			case UART_1:
				myLsrType = calShadowRegs.laserTypeA;
				break;
			case UART_2:
				myLsrType = calShadowRegs.laserTypeB;
				break;
			case UART_3:
				myLsrType = calShadowRegs.laserTypeC;
				break;
			case UART_4:
				myLsrType = calShadowRegs.laserTypeD;
				break;
			default:
				break;
		}	
	}
	else
	{
		strupr(myCmdToken,myBuf);		// convert command to upper case, store a copy, skip prefix
		strupr(myTmpBuf,myBuf);		    //convert command to upper case, store a copy
		activeOutPort=NULL;
	}		

	myTmpPtr = myCmdToken;
	myCnt = 0;	

	// truncate the input to just the "command" no spaces, New Line character, equal signs, or arguments
	while(*myTmpPtr	!= CR_KEY && *myTmpPtr != 0)
	{
		if(*myTmpPtr	== ' ')
		{
			didWeFindEndOfCmdToken = TRUE;
		 	*myTmpPtr = NULL;		// end of command token.
			break;
		}
		else if(*myTmpPtr	== '=')
		{
			didWeFindEndOfCmdToken = TRUE;
		 	*myTmpPtr = NULL;		// end of command token.
			break;
		}
		else
		{
			myTmpPtr++; //Ordinary character	
		}	

		if(myCnt++ > MAX_CMD_SIZE)		// did we exceed the size?
		{
			*myCmdToken = NULL;					// yes, toss input
		 	break;	// error shall be handled in following statements
		}
	}

	//Executes when entering laser specific like "c.?le" or global like "vort"
	if(*myTmpPtr == CR_KEY)
	{
		didWeFindCrKey = TRUE;
		*myTmpPtr = NULL;					// yes, remove the Carriage return character
	}
	
	myCmdLength = strlen(myCmdToken);

	if(myCnt > MAX_CMD_SIZE)
	{
		return;	// did not get the "CR"
	}
	else if(didWeFindEndOfCmdToken == TRUE)	// found "=" or " ", equal character or space
	{
		// is there a CR Key entered?
		myTmpPtr++;

		// search for the CR Key
		while(*myTmpPtr	!= CR_KEY && *myTmpPtr	!= 0)
		{
			myTmpPtr++;

			if(myCnt++ > MAX_CMD_SIZE)		// did we exceed the size?
			{
				*myCmdToken = NULL;					// yes, toss input
			 	break;	// error shall be handled in following statements
			}
		}

		if(*myTmpPtr == CR_KEY)
		{
			didWeFindCrKey = TRUE;
			*myTmpPtr = NULL;					// yes, remove the Carriage return character
		}
		else			// did not find CR key!! No reason to parse command
			return;
	}
	else if(didWeFindCrKey == FALSE)
	{
		return;	// did not get the "CR"
	}

	while(SystemCmds[indx].cmd[0] != 0)	 	// loop through all available valid commands
	{
		myLength = strlen(SystemCmds[indx].cmd);

		if(strcmp(myCmdToken, SystemCmds[indx].cmd) == 0)	// is this a valid command?
		{
			if(SystemCmds[indx].prefixReqd==didWeFindEndOfPrefixToken)
			{
				if(SystemCmds[indx].maxArgs == 0)		 // Pass Arguments?
				{
					if(myTmpBuf[myLength] == CR_KEY)		// yes, same length
					{
						myTmpBuf[myLength] = NULL;		// change the carriage return into a NULL character

						SystemCmds[indx].cmdHandler();	//No Arguments

						cmdMatched = 1;
					}
				}
				else if(SystemCmds[indx].maxArgs == 1)		// 1 argument expected
				{
					myTmpPtr = &myTmpBuf[myLength];

					myTmpLength = myLength;

					// remove spaces
					while(*myTmpPtr == ' ')
					{
						myTmpPtr++;
						myLength++;
					}

					// is there an "=" character"
					if(*myTmpPtr == '=')
					{
						myTmpPtr++;		// move to next character
						myLength++;

						// remove spaces
						while(*myTmpPtr == ' ')
						{
							myTmpPtr++;
							myLength++;
						}
	
						// get argument/token
						arg1 = myTmpPtr;	// yes, 1 argument
	
						// find the "CR"
						while(*myTmpPtr != CR_KEY)
						{
							myTmpPtr++;
							myLength++;
						}

						*myTmpPtr = 0;	// replace the "CR" with a NULL
						myLength++;
					}
					SystemCmds[indx].cmdHandler();	// PASS TOKENS!!!
	
					cmdMatched = 1;						
				}
			}
			if(cmdMatched)
				break;
		}
		indx++;		
	}
#if 0
		// did we get a Carriage return?
		for(myCnt = 0; myCnt < (myMsgSize - charsRemoved); myCnt++)
		{
			if(myBuf[myCnt] == CR_KEY)
			{
				return;
			}
		}
#endif
		// Command was not found in table. Is it still a valid command?
		if(cmdMatched == 0)
		{
			if(isManufModeActive==TRUE)
			{
				//Forward the laser specific command even though it is unknown
				generalFrontHandler(myCmdLength);
			}
			else
				composePrompt();	
		}
}

void generalFrontHandler(unsigned char myLength)
{
	char *myTmpPtr;
	unsigned char numTokens=0;
	unsigned char myTmpLength=0;
	
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{
		//Look for '?' to determine if arg1 should be passed
		if(myCmdToken[0]=='?')
		{
			numTokens=1;	
		}
		else
		{
			numTokens=2;
		}		
		
		if(numTokens==1)		 // Pass Arguments?
		{
			if(myTmpBuf[myLength] == CR_KEY)		// yes, same length
			{
				myTmpBuf[myLength] = NULL;		// change the carriage return into a NULL character

				sprintf(outgoing, "%s\r", myCmdToken);
				sendString(activeOutPort,outgoing);
				setLaserStatus(activeOutPort);	
			}
		}
		else if(numTokens == 2)		// 1 argument expected
		{
			myTmpPtr = &myTmpBuf[myLength];

			myTmpLength = myLength;

			// remove spaces
			while(*myTmpPtr == ' ')
			{
				myTmpPtr++;
				myLength++;
			}

			// is there an "=" character"
			if(*myTmpPtr == '=')
			{
				myTmpPtr++;		// move to next character
				myLength++;

				// remove spaces
				while(*myTmpPtr == ' ')
				{
					myTmpPtr++;
					myLength++;
				}
	
				// get argument/token
				arg1 = myTmpPtr;	// yes, 1 argument
	
				// find the "CR"
				while(*myTmpPtr != CR_KEY)
				{
					myTmpPtr++;
					myLength++;
				}
	
				*myTmpPtr = 0;	// replace the "CR" with a NULL
				myLength++;
				
				sprintf(outgoing, "%s=%s\r", myCmdToken,arg1);
				sendString(activeOutPort,outgoing);
				setLaserStatus(activeOutPort);
			}
			else
			{
				if(myTmpBuf[myLength] == CR_KEY)		// yes, same length
				{
					myTmpBuf[myLength] = NULL;	
					
					//For commands with no equal sign and no argument
					sprintf(outgoing, "%s\r", myCmdToken);
					sendString(activeOutPort,outgoing);
					setLaserStatus(activeOutPort);
				}	
			}								
		}						
	}	
}	

ushort getLsrType(ushort uartSlot)
{
	ushort myType;
	
	if(uartSlot==UART_1)
		myType=calShadowRegs.laserTypeA;
	else if(uartSlot==UART_2)
		myType=calShadowRegs.laserTypeB;
	else if(uartSlot==UART_3)
        myType=calShadowRegs.laserTypeC;
	else if(uartSlot==UART_4)
		myType=calShadowRegs.laserTypeD;
	return myType;
}

void detectLasers()
{
	//Ping each laser to find out which slots are occupied.
	//Force the Stradus Prompt to be returned so the fault codes may be relayed to VersaLase Prompt without
	//querying for fault codes directly.
	
	setLaserStatus(UART_1);
	setLaserStatus(UART_2);
	setLaserStatus(UART_3);
	setLaserStatus(UART_4);
	//GLOBAL_Response_Pending=1;
	if(calShadowRegs.laserTypeA==LSR_STRADUS)
	{
		sprintf(outgoing, "prompt=1\r");
		sendString(UART_1,outgoing);
	}
	else if(calShadowRegs.laserTypeA==LSR_OXXIUS)
	{
		cmdToMatch[activeOutPort]=12;
		sprintf(outgoing, "ve?\n");
		sendString(UART_1,outgoing);
	}
	if(calShadowRegs.laserTypeB==LSR_STRADUS)
	{
		sprintf(outgoing, "prompt=1\r");
		sendString(UART_2,outgoing);
	}
	else if(calShadowRegs.laserTypeB==LSR_OXXIUS)
	{
		cmdToMatch[activeOutPort]=12;
		sprintf(outgoing, "ve?\n");
		sendString(UART_2,outgoing);
	}
	if(calShadowRegs.laserTypeC==LSR_STRADUS)
	{
		sprintf(outgoing, "prompt=1\r");
		sendString(UART_3,outgoing);
	}
	else if(calShadowRegs.laserTypeC==LSR_OXXIUS)
	{
		cmdToMatch[activeOutPort]=12;
		sprintf(outgoing, "ve?\n");
		sendString(UART_3,outgoing);
	}
	if(calShadowRegs.laserTypeD==LSR_STRADUS)
	{
		sprintf(outgoing, "prompt=1\r");
		sendString(UART_4,outgoing);
	}
	else if(calShadowRegs.laserTypeD==LSR_OXXIUS)
	{
		cmdToMatch[activeOutPort]=12;
		sprintf(outgoing, "ve?\n");
		sendString(UART_4,outgoing);
	}
}

void setLaserStatus(unsigned char laserPort)
{
	if(laserPort==UART_1)
		A_Response_Pending=1;		
	else
		if(laserPort==UART_2)
			B_Response_Pending=1;
		else
			if(laserPort==UART_3)
				C_Response_Pending=1;
			else
				if(laserPort==UART_4)
					D_Response_Pending=1;
}	
	
//Command Handler Functions**********************************************************

void cHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s=%s\r", myCmdToken,arg1);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);	
		}
		else
			if(myLsrType==LSR_OXXIUS) //Oxxius does not do constant current
			{
				composePrompt();
			}		
	}
}
//Individual
void delayHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s=%s\r", myCmdToken,arg1);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);	
		}
		else
			if(myLsrType==LSR_OXXIUS) //CDRH delay always on for Oxxius
			{
				sprintf(outgoing, "\r\n%s=1", myCmdToken);
				sendString(portToHost,outgoing);
				composePrompt();
			}		
	}	
}

//Internal
void echoHandler(void)
{
	if(*arg1 == '1')
	{
		sysCfgShadowRegs.Echo=1;
		saveSysCfgDelta(EE_AREA_SYS_CFG);
		sprintf(outgoing, "\r\n%s=1", myCmdToken);
		sendString(portToHost,outgoing);
	}
	else if(*arg1 == '0')
	{
		sysCfgShadowRegs.Echo = 0;
		saveSysCfgDelta(EE_AREA_SYS_CFG);
		sprintf(outgoing, "\r\n%s=0", myCmdToken);
		sendString(portToHost,outgoing);
	}
	composePrompt();
}

//Individual
void epcHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s=%s\r", myCmdToken,arg1);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
			composePrompt();
	}
}
//Individual
void leHandler(void)
{

	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s=%s\r", myCmdToken,arg1);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS)
		{	
			if(*arg1=='1')
				cmdToMatch[activeOutPort]=1; //On
			else
				if(*arg1=='0')
					cmdToMatch[activeOutPort]=11; //Off		
			sprintf(outgoing, "dl %s\n",arg1);		
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}	
		else
			composePrompt();				
	}	
}
//Individual
void lpHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{
		if(myLsrType==LSR_STRADUS)
		{	
			sprintf(outgoing, "%s=%s\r", myCmdToken,arg1);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
			composePrompt();
	}	
}
//Internal
void promptHandler(void)
{
	if(*arg1 == '1')
	{
		sysCfgShadowRegs.Prompt=1;
		saveSysCfgDelta(EE_AREA_SYS_CFG);
		sprintf(outgoing, "\r\n%s=1", myCmdToken);
		sendString(portToHost,outgoing);
	}
	else if(*arg1 == '0')
	{
		sysCfgShadowRegs.Prompt = 0;
		saveSysCfgDelta(EE_AREA_SYS_CFG);
		sprintf(outgoing, "\r\n%s=0", myCmdToken);
		sendString(portToHost,outgoing);
	}
	composePrompt();
}
//Individual
void tecHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s=%s\r", myCmdToken,arg1);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}
}
//Individual
void vobtsHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s=%s\r", myCmdToken,arg1);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}	
}

void bypassHandler(void)
{	
	if(isManufModeActive==TRUE)
	{
		shadowRegs.Bypass=1; //Volatile setting
		sprintf(outgoing, "\r\nBYPASS=1");
		sendString(portToHost,outgoing);
	}	
	composePrompt();	
}

//Individual
void pulHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s=%s\r", myCmdToken,arg1);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}	
}

//Individual
void ppHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s=%s\r", myCmdToken,arg1);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}	
}
	
//Global-Special Case
void q_bptHandler(void)
{
	if(theChosenOne>0)
	{
		sprintf(outgoing, "%s\r", myCmdToken);
		sendString(theChosenOne,outgoing);
		setLaserStatus(theChosenOne); //query a laser present in the system
	}
	else
		composePrompt();
}

void q_cHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing,"%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);	
		}
		else
			if(myLsrType==LSR_OXXIUS) //Oxxius does not do constant current
			{
				composePrompt();
			}		
	}
}

void q_ccHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing,"%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);	
		}
		else
			if(myLsrType==LSR_OXXIUS) //Oxxius does not do constant current
			{
				composePrompt();
			}		
	}
}

//Individual
void q_ctHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS) //Not supported
		{
			composePrompt();
		}
		else
			if(myLsrType==LSR_OXXIUS)
			{
				cmdToMatch[activeOutPort]=10;
				sprintf(outgoing, "et?\n");
				sendString(activeOutPort,outgoing);
				setLaserStatus(activeOutPort);
			}
	}
}

//Individual
void q_delayHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
			if(myLsrType==LSR_OXXIUS) //CDRH delay always on for Oxxius
			{
				sprintf(outgoing, "\r\n%s=1", myCmdToken);
				sendString(portToHost,outgoing);
				composePrompt();
			}
	}
}
//Individual
void q_epcHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}
}
//Individual
void q_fcHandler(void)
{
	char dlCmd[MAX_CMD_SIZE]={"dl?"};

	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS)
		{
			cmdToMatch[activeOutPort]=2;
			sprintf(outgoing, "%s\n",dlCmd);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
			composePrompt();
	}	
}
//Individual
void q_fdHandler(void)
{
	char dlCmd[MAX_CMD_SIZE]={"dl?"};

	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS)
		{
			cmdToMatch[activeOutPort]=3;
			sprintf(outgoing, "%s\n",dlCmd);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}	
}
//Individual
void q_fpHandler(void) 
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}	
}
//Individual
void q_fvHandler(void)
{
	char veCmd[MAX_CMD_SIZE]={"ve?"};

	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS)
		{
			cmdToMatch[activeOutPort]=8;
			sprintf(outgoing, "%s\n",veCmd);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}
}
//Internal
void q_spvHandler(void)
{
	sprintf(outgoing, "\r\n%s=%s", myCmdToken, PROTOCOL_VER);
	sendString(portToHost,outgoing);
	composePrompt();
}
//Internal
void q_sfvHandler(void)
{
	sprintf(outgoing, "\r\n%s=%s",myCmdToken,SW_VERSION);
	sendString(portToHost,outgoing);
	composePrompt();
}

//Internal-"GUI1"-Concatenated string of parameters to be read by GUI only
void q_gui1Handler(void)
{
	//Returns LaserTypeA*LaserTypeB*LaserTypeC*LaserTypeD*?SFV*?SPV
	sprintf(outgoing, "\r\n%s=%d*%d*%d*%d*%s*%s",myCmdToken,calShadowRegs.laserTypeA,calShadowRegs.laserTypeB,calShadowRegs.laserTypeC,calShadowRegs.laserTypeD,SW_VERSION,PROTOCOL_VER);
	sendString(portToHost,outgoing);
	composePrompt();
}

//Internal
void q_hHandler(void)
{
	int myIndex=0;
	
	if(portToHost==EXT_UART)
	{
		sprintf(outgoing, "\r\n\r\n");
		sendString(portToHost,outgoing);
		while(helpTable[myIndex][0] != 0)
		{
			delayMs(10);
			sprintf(outgoing, "%s\r\n", &helpTable[myIndex++][0]);	
			sendString(portToHost,outgoing);	
		}
		composePrompt();
	}
	else
		if(portToHost==USB_PORT)
		{				
			sprintf(outgoing, "\r\n\r\n");
			sendString(portToHost,outgoing);
			while(helpTable[myIndex][0] != 0)
			{
				delayMs(10);
				sprintf(outgoing, "%s\r\n", &helpTable[myIndex++][0]);
				sendString(portToHost,outgoing);		
			}
			composePrompt();					
		}	
}
	
//Global-Get from a laser that is present in the system (no prefix)
void q_ilHandler(void)
{
	if(theChosenOne>0)
	{
		sprintf(outgoing, "%s\r", myCmdToken);
		sendString(theChosenOne,outgoing);
		setLaserStatus(theChosenOne); //query a laser present in the system
	}
	else
		composePrompt();
}

//Individual
void q_lcHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}	
}
//Individual
void q_leHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS)
		{
			cmdToMatch[activeOutPort]=5;
			sprintf(outgoing, "dl?\n");
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}
}
//Individual
void q_lhHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS)
		{
			cmdToMatch[activeOutPort]=7;
			sprintf(outgoing, "tm?\n");
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}	
}
//Individual
void q_liHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS) //Get from laser
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS) //Get from Versa-Lase EEPROM
		{
			if(activeOutPort==UART_1)
			{
				sprintf(outgoing, "\r\nA.%s=%s, %s, %dnm, %dmW, %s", myCmdToken, calShadowRegs.lsrSerialNum_A, calShadowRegs.lsrPartNum_A, calShadowRegs.nomWaveLen_A, calShadowRegs.nomLsrPwr_A, calShadowRegs.lsrBeamShape_A);
			}
			else if(activeOutPort==UART_2)
			{
				sprintf(outgoing, "\r\nB.%s=%s, %s, %dnm, %dmW, %s", myCmdToken, calShadowRegs.lsrSerialNum_B, calShadowRegs.lsrPartNum_B, calShadowRegs.nomWaveLen_B, calShadowRegs.nomLsrPwr_B, calShadowRegs.lsrBeamShape_B);
			}
			else if(activeOutPort==UART_3)
			{
				sprintf(outgoing, "\r\nC.%s=%s, %s, %dnm, %dmW, %s", myCmdToken, calShadowRegs.lsrSerialNum_C, calShadowRegs.lsrPartNum_C, calShadowRegs.nomWaveLen_C, calShadowRegs.nomLsrPwr_C, calShadowRegs.lsrBeamShape_C);
			}
			else if(activeOutPort==UART_4)
				sprintf(outgoing, "\r\nD.%s=%s, %s, %dnm, %dmW, %s", myCmdToken, calShadowRegs.lsrSerialNum_D, calShadowRegs.lsrPartNum_D, calShadowRegs.nomWaveLen_D, calShadowRegs.nomLsrPwr_D, calShadowRegs.lsrBeamShape_D);

			sendString(portToHost,outgoing);
			composePrompt();
		}
		else
		{
			composePrompt();
		}
	}	
}
//Individual
void q_lpHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS)
		{
			cmdToMatch[activeOutPort]=6;
			sprintf(outgoing, "ip?\n");
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}	
}
//Individual
void q_lpsHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS)
		{
			if(activeOutPort==UART_1)
			{
				sprintf(outgoing, "\r\nA.%s=%d", myCmdToken, calShadowRegs.nomLsrPwr_A);
			}
			else if(activeOutPort==UART_2)
			{
				sprintf(outgoing, "\r\nB.%s=%d", myCmdToken, calShadowRegs.nomLsrPwr_B);
			}
			else if(activeOutPort==UART_3)
			{	
				sprintf(outgoing, "\r\nC.%s=%d", myCmdToken, calShadowRegs.nomLsrPwr_C);
			}
			else if(activeOutPort==UART_4)
			{
				sprintf(outgoing, "\r\nD.%s=%d", myCmdToken, calShadowRegs.nomLsrPwr_D);
			}			
			sendString(portToHost,outgoing);
			composePrompt();
		}
		else
		{
			composePrompt();
		}
	}	
}
//Individual
void q_lwHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS)
		{
			if(activeOutPort==UART_1)
			{
				sprintf(outgoing, "\r\nA.%s=%d", myCmdToken, calShadowRegs.nomWaveLen_A);
			}
			else if(activeOutPort==UART_2)
			{
				sprintf(outgoing, "\r\nB.%s=%d", myCmdToken, calShadowRegs.nomWaveLen_B);
			}
			else if(activeOutPort==UART_3)
			{	
				sprintf(outgoing, "\r\nC.%s=%d", myCmdToken, calShadowRegs.nomWaveLen_C);
			}
			else if(activeOutPort==UART_4)
			{
				sprintf(outgoing, "\r\nD.%s=%d", myCmdToken, calShadowRegs.nomWaveLen_D);
			}			
			sendString(portToHost,outgoing);
			composePrompt();
		}
		else
		{
			composePrompt();
		}
	}	
}
//Individual
void q_maxpHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS)
		{
			if(activeOutPort==UART_1)
			{
				sprintf(outgoing, "\r\nA.%s=%4.1f", myCmdToken, calShadowRegs.maxPwrLsr_A);
			}
			else if(activeOutPort==UART_2)
			{
				sprintf(outgoing, "\r\nB.%s=%4.1f", myCmdToken, calShadowRegs.maxPwrLsr_B);
			}
			else if(activeOutPort==UART_3)
			{	
				sprintf(outgoing, "\r\nC.%s=%4.1f", myCmdToken, calShadowRegs.maxPwrLsr_C);
			}
			else if(activeOutPort==UART_4)
			{
				sprintf(outgoing, "\r\nD.%s=%4.1f", myCmdToken, calShadowRegs.maxPwrLsr_D);
			}			
			sendString(portToHost,outgoing);
			composePrompt();
		}
		else
		{
			composePrompt();
		}
	}	
}
//Individual
void q_obtHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS)
		{
			cmdToMatch[activeOutPort]=4;
			sprintf(outgoing, "bt?\n");
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}		
}
//Individual
void q_obtsHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}	
}
//Individual
void q_rpHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS) //Get from laser
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else if(myLsrType==LSR_OXXIUS) //Get from Versa-Lase EEPROM
		{
			if(activeOutPort==UART_1)
			{
				sprintf(outgoing, "\r\nA.%s=%d", myCmdToken,calShadowRegs.nomLsrPwr_A);
			}
			else if(activeOutPort==UART_2)
			{
				sprintf(outgoing, "\r\nB.%s=%d", myCmdToken,calShadowRegs.nomLsrPwr_B);
			}
			else if(activeOutPort==UART_3)
			{
				sprintf(outgoing, "\r\nC.%s=%d", myCmdToken,calShadowRegs.nomLsrPwr_C);
			}
			else if(activeOutPort==UART_4)
				sprintf(outgoing, "\r\nD.%s=%d", myCmdToken,calShadowRegs.nomLsrPwr_D);

			sendString(portToHost,outgoing);
			composePrompt();
		}
		else
		{
			composePrompt();
		}
	}	
}

//Individual
void q_tecHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}	
}

//Individual
void q_pulHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}	
}

//Individual
void q_ppHandler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}	
}

//Individual
void q_op1Handler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}
}

//Individual
void q_op2Handler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}
}

//Individual
void q_op3Handler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}
}

//Individual
void q_op4Handler(void)
{
	if(myPrefixToken!=NULL) //Typed in with a prefix
	{	
		if(myLsrType==LSR_STRADUS)
		{
			sprintf(outgoing, "%s\r", myCmdToken);
			sendString(activeOutPort,outgoing);
			setLaserStatus(activeOutPort);
		}
		else
		{
			composePrompt();
		}
	}
}

//&&& Calibration Commands &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
void vortHandler(void)
{
	isManufModeActive=TRUE;

	sprintf(outgoing, "\r\nMfr Mode");
	sendString(portToHost,outgoing);
	composePrompt();
}

void burnHandler(void)
{
	if(isManufModeActive==TRUE)
	{
		forceSysCfgDefaults();
		sprintf(outgoing, "\r\nDEFAULT VALUES SET");
		sendString(portToHost,outgoing);
	}	
	composePrompt();
}

void dsysCfgHandler(void)
{
	if(isManufModeActive==TRUE)
	{
			sprintf(outgoing,"\r\nCONFIG VALUES:");
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing, "\r\nVersion: %d",sysCfgShadowRegs.version);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nEcho: %d",sysCfgShadowRegs.Echo);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nPrompt: %d",sysCfgShadowRegs.Prompt);
			sendString(portToHost,outgoing);
			delayMs(5);

			sprintf(outgoing,"\r\nCAL VALUES:");
			sendString(portToHost,outgoing);	
			delayMs(5);
			sprintf(outgoing,"\r\nCalVersion: %f",calShadowRegs.calVersion);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLsr Type A*B*C*D: %d*%d*%d*%d",calShadowRegs.laserTypeA,calShadowRegs.laserTypeB,calShadowRegs.laserTypeC,calShadowRegs.laserTypeD);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLsr MaxP A*B*C*D: %4.2f*%4.2f*%4.2f*%4.2f",calShadowRegs.maxPwrLsr_A,calShadowRegs.maxPwrLsr_B,calShadowRegs.maxPwrLsr_C,calShadowRegs.maxPwrLsr_D);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLsr NomP A*B*C*D: %d*%d*%d*%d",calShadowRegs.nomLsrPwr_A,calShadowRegs.nomLsrPwr_B,calShadowRegs.nomLsrPwr_C,calShadowRegs.nomLsrPwr_D);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLsr NomWave A*B*C*D: %d*%d*%d*%d",calShadowRegs.nomWaveLen_A,calShadowRegs.nomWaveLen_B,calShadowRegs.nomWaveLen_C,calShadowRegs.nomWaveLen_D);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLaser PN A: %s",calShadowRegs.lsrPartNum_A);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLaser PN B: %s",calShadowRegs.lsrPartNum_B);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLaser PN C: %s",calShadowRegs.lsrPartNum_C);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLaser PN D: %s",calShadowRegs.lsrPartNum_D);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLaser SN A: %s",calShadowRegs.lsrSerialNum_A);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLaser SN B: %s",calShadowRegs.lsrSerialNum_B);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLaser SN C: %s",calShadowRegs.lsrSerialNum_C);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLaser SN D: %s",calShadowRegs.lsrSerialNum_D);
			sendString(portToHost,outgoing);
			delayMs(5);
			sprintf(outgoing,"\r\nLaser Beam Shape A*B*C*D: %s*%s*%s*%s",calShadowRegs.lsrBeamShape_A,calShadowRegs.lsrBeamShape_B,calShadowRegs.lsrBeamShape_C,calShadowRegs.lsrBeamShape_D);
			sendString(portToHost,outgoing);
			delayMs(5);
	}
	composePrompt();
}	


void calStradusMfrModeHandler()
{
	if(isManufModeActive==TRUE)
	{
		if(myPrefixToken!=NULL) //Typed in with a prefix
		{	
			if(myLsrType==LSR_STRADUS)
			{
				sprintf(outgoing, "%s\r", myCmdToken);
				sendString(activeOutPort,outgoing);
				setLaserStatus(activeOutPort);
			}
			else
			{
				composePrompt();
			}
		}
	}
	else	
		composePrompt();		
}	

void calMaxPwrLsrHandler(void)
{
	double tempMaxP;

	if(isManufModeActive==TRUE)
	{
		if(myPrefixToken!=NULL) //Typed in with a prefix
		{
			tempMaxP=atof(arg1);	
			if(tempMaxP>=0 && tempMaxP<=1000)
			{
				if(activeOutPort==UART_1)
				{
					calShadowRegs.maxPwrLsr_A=tempMaxP;
					sprintf(outgoing, "\r\nA.%s=%4f",myCmdToken,tempMaxP);
				}
				else if(activeOutPort==UART_2)
				{
					calShadowRegs.maxPwrLsr_B=tempMaxP;	
					sprintf(outgoing, "\r\nB.%s=%4f",myCmdToken,tempMaxP);
				}
				else if(activeOutPort==UART_3)
				{
					calShadowRegs.maxPwrLsr_C=tempMaxP;
					sprintf(outgoing, "\r\nC.%s=%4f",myCmdToken,tempMaxP);
				}
				else if(activeOutPort==UART_4)
				{
					calShadowRegs.maxPwrLsr_D=tempMaxP;
					sprintf(outgoing, "\r\nD.%s=%4f",myCmdToken,tempMaxP);
				}
				saveSysCfgDelta(EE_AREA_CAL);
				sendString(portToHost,outgoing);
			}
		}				
	}
	composePrompt();
}

void calNomLsrPwrHandler(void)
{
	short tempNomP;

	if(isManufModeActive==TRUE)
	{
		if(myPrefixToken!=NULL) //Typed in with a prefix
		{
			tempNomP=atoi(arg1);			
			if(tempNomP>=0 && tempNomP<=1000)
			{
				if(activeOutPort==UART_1)
				{
					calShadowRegs.nomLsrPwr_A=tempNomP;
					sprintf(outgoing, "\r\nA.%s=%4d",myCmdToken,tempNomP);
				}
				else if(activeOutPort==UART_2)
				{
					calShadowRegs.nomLsrPwr_B=tempNomP;	
					sprintf(outgoing, "\r\nB.%s=%4d",myCmdToken,tempNomP);
				}
				else if(activeOutPort==UART_3)
				{
					calShadowRegs.nomLsrPwr_C=tempNomP;
					sprintf(outgoing, "\r\nC.%s=%4d",myCmdToken,tempNomP);
				}
				else if(activeOutPort==UART_4)
				{
					calShadowRegs.nomLsrPwr_D=tempNomP;
					sprintf(outgoing, "\r\nD.%s=%4d",myCmdToken,tempNomP);
				}
				saveSysCfgDelta(EE_AREA_CAL);
				sendString(portToHost,outgoing);
			}	
		}			
	}
	composePrompt();
}

void calNomWaveLenHandler(void)
{
	// measWaveLen_A;
	short tempNomWav;

	if(isManufModeActive==TRUE)
	{
		if(myPrefixToken!=NULL) //Typed in with a prefix
		{
			tempNomWav=atoi(arg1);		
			if(tempNomWav>=0 && tempNomWav<=1000)
			{
				if(activeOutPort==UART_1)
				{
					calShadowRegs.nomWaveLen_A=tempNomWav;
					sprintf(outgoing, "\r\nA.%s=%4d",myCmdToken,tempNomWav);
				}
				else if(activeOutPort==UART_2)
				{
					calShadowRegs.nomWaveLen_B=tempNomWav;
					sprintf(outgoing, "\r\nB.%s=%4d",myCmdToken,tempNomWav);
				}	
				else if(activeOutPort==UART_3)
				{
					calShadowRegs.nomWaveLen_C=tempNomWav;
					sprintf(outgoing, "\r\nC.%s=%4d",myCmdToken,tempNomWav);
				}
				else if(activeOutPort==UART_4)
				{
					calShadowRegs.nomWaveLen_D=tempNomWav;
					sprintf(outgoing, "\r\nD.%s=%4d",myCmdToken,tempNomWav);
				}
				saveSysCfgDelta(EE_AREA_CAL);
				sendString(portToHost,outgoing);
			}	
		}			
	}
	composePrompt();
}

void calPartNumHandler(void)
{
	if(isManufModeActive==TRUE)
	{
		if(myPrefixToken!=NULL) //Typed in with a prefix
		{
			if(strlen(arg1)<11)
			{
				if(activeOutPort==UART_1)
				{
					strcpy(calShadowRegs.lsrPartNum_A,arg1);
					sprintf(outgoing, "\r\nA.%s=%s",myCmdToken,arg1);
				}
				else if(activeOutPort==UART_2)
				{
					strcpy(calShadowRegs.lsrPartNum_B,arg1);
					sprintf(outgoing, "\r\nB.%s=%s",myCmdToken,arg1);
				}	
				else if(activeOutPort==UART_3)
				{
					strcpy(calShadowRegs.lsrPartNum_C,arg1);
					sprintf(outgoing, "\r\nC.%s=%s",myCmdToken,arg1);
				}
				else if(activeOutPort==UART_4)
				{
					strcpy(calShadowRegs.lsrPartNum_D,arg1);
					sprintf(outgoing, "\r\nD.%s=%s",myCmdToken,arg1);
				}
				saveSysCfgDelta(EE_AREA_CAL);
				sendString(portToHost,outgoing);
			}	
		}			
	}
	composePrompt();
	

}
void calSerNumHandler(void)
{
	if(isManufModeActive==TRUE)
	{
		if(myPrefixToken!=NULL) //Typed in with a prefix
		{
			if(strlen(arg1)<11)
			{
				if(activeOutPort==UART_1)
				{
					strcpy(calShadowRegs.lsrSerialNum_A,arg1);
					sprintf(outgoing, "\r\nA.%s=%s",myCmdToken,arg1);
				}
				else if(activeOutPort==UART_2)
				{
					strcpy(calShadowRegs.lsrSerialNum_B,arg1);	
					sprintf(outgoing, "\r\nB.%s=%s",myCmdToken,arg1);
				}
				else if(activeOutPort==UART_3)
				{
					strcpy(calShadowRegs.lsrSerialNum_C,arg1);
					sprintf(outgoing, "\r\nC.%s=%s",myCmdToken,arg1);
				}
				else if(activeOutPort==UART_4)
				{
					strcpy(calShadowRegs.lsrSerialNum_D,arg1);
					sprintf(outgoing, "\r\nD.%s=%s",myCmdToken,arg1);
				}
				saveSysCfgDelta(EE_AREA_CAL);
				sendString(portToHost,outgoing);
			}
		}
				
	}
	composePrompt();

}
void calBeamShapeHandler(void)
{
	if(isManufModeActive==TRUE)
	{
		if(myPrefixToken!=NULL) //Typed in with a prefix
		{
			if(strlen(arg1)<4)
			{
				if(activeOutPort==UART_1)
				{
					strcpy(calShadowRegs.lsrBeamShape_A,arg1);
					sprintf(outgoing, "\r\nA.%s=%s",myCmdToken,arg1);
				}
				else if(activeOutPort==UART_2)
				{
					strcpy(calShadowRegs.lsrBeamShape_B,arg1);	
					sprintf(outgoing, "\r\nB.%s=%s",myCmdToken,arg1);
				}
				else if(activeOutPort==UART_3)
				{
					strcpy(calShadowRegs.lsrBeamShape_C,arg1);
					sprintf(outgoing, "\r\nC.%s=%s",myCmdToken,arg1);
				}
				else if(activeOutPort==UART_4)
				{
					strcpy(calShadowRegs.lsrBeamShape_D,arg1);
					sprintf(outgoing, "\r\nD.%s=%s",myCmdToken,arg1);
				}
				saveSysCfgDelta(EE_AREA_CAL);
				sendString(portToHost,outgoing);
			}
		}						
	}
	composePrompt();

}

//Sets laser type for each laser, requires reboot afterwords before communication with laser is possible
void calTypeHandler(void)
{
	ushort myType;

	if(isManufModeActive==TRUE)
	{
		if(myPrefixToken!=NULL) //Typed in with a prefix
		{
			myType=atoi(arg1);			
			if(myType>=0 && myType<=15)
			{
				if(activeOutPort==UART_1)
				{
					calShadowRegs.laserTypeA = myType;
					sprintf(outgoing, "\r\nA.%s=%d",myCmdToken,myType);
				}
				else if(activeOutPort==UART_2)
				{
					calShadowRegs.laserTypeB = myType;	
					sprintf(outgoing, "\r\nB.%s=%d",myCmdToken,myType);
				}
				else if(activeOutPort==UART_3)
				{
					calShadowRegs.laserTypeC = myType;
					sprintf(outgoing, "\r\nC.%s=%d",myCmdToken,myType);
				}
				else if(activeOutPort==UART_4)
				{
					calShadowRegs.laserTypeD = myType;
					sprintf(outgoing, "\r\nD.%s=%d",myCmdToken,myType);
				}						
				saveSysCfgDelta(EE_AREA_CAL);				
				sendString(portToHost,outgoing);		
			}
		}
	}
	composePrompt();
}
	

void q_calTypeHandler(void)
{
	if(isManufModeActive==TRUE)
	{
		if(myPrefixToken!=NULL) //Typed in with a prefix
		{
			if(activeOutPort==UART_1)
			{		
				sprintf(outgoing, "\r\nA.%s=%d",myCmdToken,calShadowRegs.laserTypeA);
			}
			else if(activeOutPort==UART_2)
			{	
				sprintf(outgoing, "\r\nB.%s=%d",myCmdToken,calShadowRegs.laserTypeB);
			}
			else if(activeOutPort==UART_3)
			{
				sprintf(outgoing, "\r\nC.%s=%d",myCmdToken,calShadowRegs.laserTypeC);
			}
			else if(activeOutPort==UART_4)
			{
				sprintf(outgoing, "\r\nD.%s=%d",myCmdToken,calShadowRegs.laserTypeD);
			}
			sendString(portToHost,outgoing);
		}
	}
	composePrompt();
}

//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&


void invalidCommand()
{
	sprintf(outgoing, "\nValue Out Of Range");
	sendString(portToHost,outgoing);
}	

void composePrompt()//unsigned char myPort)
{	
	if(configuring==FALSE)
	{
		if(sysCfgShadowRegs.Prompt)
		{
			//sprintf(myEndingToken,"\r\nStradus:%s:%s:%s:%s> ",shadowRegs.A_qFC,shadowRegs.B_qFC,shadowRegs.C_qFC,shadowRegs.D_qFC);
			sprintf(myEndingToken,"\r\nStradus> ");
		}
		else
		{
			sprintf(myEndingToken,"\r\n");	
		}
	}
			
	if(portToHost==USB_PORT)
	{
		//Combine parsed cmd response and prompt and place in USB IN buffer
		sendString(USB_PORT,myEndingToken);
		//used to be sendString(USB_PORT_FINAL,myEndingToken);
	}
	else if(portToHost==EXT_UART)
	{
		//Parsed cmd response already sent to host on RS-232, just send prompt
		sendString(portToHost,myEndingToken);	
	}	
	else
	{
		portToHost=EXT_UART;
		sendString(portToHost,myEndingToken);
	}
	
	memset(outgoing,0,sizeof(outgoing)); //Clear outgoing buffer to 0x00's
	memset(myEndingToken,0,sizeof(myEndingToken));
}
	




	
	
	

