/*


----------------------------------------------------------------------------
 *      Name:    usbvendreq.c
 *      Purpose: USB vendor request handler
 *      Project: */

#include "Usb/usbvendreq.h"
#include "Usb/usb_device.h"
#include <string.h>

#define FALSE 0
#define TRUE 1

volatile unsigned int UsbOutPending = 0x0000;
//static BYTE UsbOutBuffer[MSG_SZ];
//static WORD UsbOutSize;

volatile unsigned int UsbInPending = 0x0000;
//static BYTE UsbInBuffer[MSG_SZ];
//static WORD UsbInSize;

unsigned char noDataResponse = 0xFF;
unsigned int isGuiCommunicatingToVort = FALSE;

unsigned char myUsbOutBuf[64] = {0};
unsigned char myUsbInBuf[64] = {0};
unsigned char myUsbInTmpBuf[64];
unsigned char myUsbOutTmpBuf[64];
short int isUsbOutDataAvailable = FALSE;
unsigned char wrQSize = 0;

/*
 * Set up where to store the OUT data
 * NOTE: Called from within USB ISR, consider mutual exclusion issues.
 */
unsigned int VendorRequestOutSetupHandler (void)
{
	unsigned char myCnt;

	if(SetupPkt.wLength > MSG_SZ) 
        return (FALSE);

	switch (SetupPkt.bRequest)
	{
		case VENDREQ_SET_COMMAND:			
			memset(myUsbOutBuf, 0, sizeof(myUsbOutBuf)); //Clears all myUsbOutBuff elements to 0x00
			//EP0Data.pData = (unsigned char *) &myUsbOutBuf;
			//wrQSize = EP0Data.Count;	
			outPipes[0].pDst.bRam = (BYTE*)&myUsbOutBuf; //pDst.bRam is a pointer to the base where data will start
			myCnt=64;	
			outPipes[0].wCount.Val=myCnt;
			//memcpy(myUsbOutBuf,myUsbOutTmpBuf,64); //dest,src												
			break;
		case VENDREQ_SET_RESPONSERECEIVED:
			memset(myUsbInTmpBuf, 0, sizeof(myUsbInTmpBuf)); //Clears all myUsbInTmpBuf elements to 0x00	
			UsbInPending=0x0000;
			break;
		default:
			return (FALSE);	// Request not handled
	}

	return (TRUE);		// Request handled
}

/*
 * Handles USB OUT vendor requests.
 * NOTE: Called from within USB ISR, consider mutual exclusion issues.
 */
unsigned int VendorRequestOutHandler (void)
{
	switch (SetupPkt.bRequest)
	{
		case VENDREQ_SET_COMMAND:
			isUsbOutDataAvailable = TRUE;
			//writeQ(USB_Q, IN_Q, (uchar *) myUsbOutBuf, wrQSize);
			memcpy(myUsbOutTmpBuf,myUsbOutBuf,64); //dest,src	
			UsbOutPending=0x0001; //OUT data is ready to be parsed in while(1) loop
			break;

		case VENDREQ_SET_RESPONSERECEIVED:
			break;
			
		default:
			return (FALSE);	// Request not handled
	}

	return (TRUE);		// Request handled
}


/*
 * Handles USB IN vendor requests.
 * NOTE: Called from within USB ISR, consider mutual exclusion issues.
 */
unsigned int VendorRequestInSetupHandler (void)
{
	switch (SetupPkt.bRequest)
	{	
 		case VENDREQ_GET_RESPONSE:
 			//For Stradus Firmware:
			//memset(myUsbInBuf, 0, sizeof(myUsbInBuf));
			//memcpy(myUsbInBuf, myUsbInTmpBuf, strlen(myUsbInTmpBuf));
			//EP0Data.pData =	(unsigned char *) myUsbInBuf;
			//EP0Data.Count = strlen(myUsbInBuf);
			//sprintf(myUsbInTmpBuf,"\r\nLE=1\r\nStradus:0> ");
			inPipes[0].pSrc.bRam = (BYTE*) myUsbInTmpBuf;
			inPipes[0].wCount.Val = strlen((char*)myUsbInTmpBuf);
			break;

		case VENDREQ_GET_RESPONSEPENDING:
		    //For Stradus Firmware
			//EP0Data.pData = (unsigned char *) &UsbInPending;
			//EP0Data.Count = 1;
			inPipes[0].pSrc.bRam = (BYTE*)&UsbInPending;
			inPipes[0].wCount.Val = 1;			
			isGuiCommunicatingToVort++;
			break;
		default:
			return (FALSE);	// Request not handled
	}

	return (TRUE);		// Request handled
}

/*
 * Handles USB IN vendor requests.
 * NOTE: Called from within USB ISR, consider mutual exclusion issues.
 */
unsigned int VendorRequestInHandler (void)
{
	switch (SetupPkt.bRequest)
	{
 		case VENDREQ_GET_RESPONSE:
			break;
		case VENDREQ_GET_RESPONSEPENDING:
			//EP0Data.Count = 1;
			break;	
		default:
			return (FALSE);	// Request not handled
	}

	return (TRUE);		// Request handled
}
